#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2025 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import sys

import KStatusNotifierItem
from PySide6.QtCore import QObject, QPoint, Qt
from PySide6.QtGui import QColor, QIcon, QPixmap
from PySide6.QtWidgets import QApplication, QMenu


class StatusNotifierTest(QObject):

    def __init__(self, parent=None) -> None:
        super().__init__(parent)

        self.system_notifier = KStatusNotifierItem.KStatusNotifierItem("org.kde.statusnotifiertest", self)
        self.system_notifier.setCategory(KStatusNotifierItem.KStatusNotifierItem.ItemCategory.Communications)
        icon = self.create_icon(Qt.red)
        self.system_notifier.setIconByPixmap(icon)

        self.system_notifier.setStatus(KStatusNotifierItem.KStatusNotifierItem.ItemStatus.Active)
        self.system_notifier.setToolTipTitle("StatusNotifierTest")
        self.system_notifier.setTitle("StatusNotifierTest")
        self.system_notifier.setToolTipSubTitle("Some explanation from the beach.")

        menu = QMenu()
        icon = self.create_icon(QColor(0, 255, 0))  # green
        action = menu.addAction(icon, "NeedsAttention")
        action.triggered.connect(self.action_triggered)
        icon = self.create_icon(QColor(255, 85, 255))  # purple
        action = menu.addAction(icon, "Active")
        action.triggered.connect(self.action_triggered)

        sub_menu = QMenu("Sub Menu")
        sub_menu.setIcon(self.create_icon(QColor(85, 0, 255)))
        icon = self.create_icon(QColor(255, 255, 0))  # yellow
        sub_action = sub_menu.addAction(icon, "Passive")
        sub_action.triggered.connect(self.action_triggered)
        menu.addMenu(sub_menu)

        self.system_notifier.setContextMenu(menu)

        self.system_notifier.activateRequested.connect(self.activate_requested)
        self.system_notifier.secondaryActivateRequested.connect(self.secondary_activate_requested)
        self.system_notifier.scrollRequested.connect(self.scroll_requested)

    @staticmethod
    def create_icon(color) -> QIcon:
        pixmap = QPixmap(16, 16)
        pixmap.fill(color)
        return QIcon(pixmap)

    def action_triggered(self) -> None:
        text = self.sender().text()
        print(text, flush=True)

        if text == "NeedsAttention":
            self.system_notifier.setStatus(KStatusNotifierItem.KStatusNotifierItem.ItemStatus.NeedsAttention)
            self.system_notifier.setIconByPixmap(self.create_icon(Qt.blue))
        elif text == "Active":
            self.system_notifier.setStatus(KStatusNotifierItem.KStatusNotifierItem.ItemStatus.Active)
            self.system_notifier.setIconByPixmap(self.create_icon(Qt.red))
        elif text == "Passive":
            self.system_notifier.setStatus(KStatusNotifierItem.KStatusNotifierItem.ItemStatus.Passive)
        else:
            raise RuntimeError("Unexpected action triggered")

    def activate_requested(self, active: bool, pos: QPoint) -> None:
        print("Activated", flush=True)

    def secondary_activate_requested(self, pos: QPoint) -> None:
        print("SecondaryActivated", flush=True)

    def scroll_requested(self, delta: int, orientation: Qt.Orientation) -> None:
        direction = "Horizontally" if orientation == Qt.Horizontal else "Vertically"
        print(f"Scrolled by {delta} {direction}", flush=True)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    test = StatusNotifierTest()
    sys.exit(app.exec())



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import logging
import os
import subprocess
import sys
import tempfile
import threading
import unittest
from typing import Final

import gi

gi.require_version('GdkPixbuf', '2.0')
from gi.repository import GdkPixbuf, Gio, GLib

sys.path.append(os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, "utils"))
from GLibMainLoopThread import GLibMainLoopThread

BIN_PATH: Final = os.path.join(os.getenv("CMAKE_BINARY_DIR", "/usr/bin/"), "plasma-apply-wallpaperimage")
DBUS_XML_PATH: Final = os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, os.pardir, "shell/dbus/org.kde.PlasmaShell.xml")


class OrgKdePlasmashell:
    """
    D-Bus interfaces for org.kde.plasmashell
    """

    BUS_NAME: Final = "org.kde.plasmashell"
    OBJECT_PATH: Final = "/PlasmaShell"
    INTERFACE_NAME: Final = "org.kde.PlasmaShell"

    connection: Gio.DBusConnection

    def __init__(self) -> None:
        self.reg_id: int = 0
        self.registered_event = threading.Event()
        self.owner_id: int = Gio.bus_own_name(Gio.BusType.SESSION, self.BUS_NAME, Gio.BusNameOwnerFlags.NONE, self.on_bus_acquired, None, None)
        assert self.owner_id > 0

        self.script: str = ""
        self.script_set_event = threading.Event()

    def quit(self) -> None:
        self.connection.unregister_object(self.reg_id)
        Gio.bus_unown_name(self.owner_id)
        self.connection.flush_sync(None)

    def on_bus_acquired(self, connection: Gio.DBusConnection, name: str, *args) -> None:
        """
        Interface is ready, now register objects.
        """
        self.connection = connection
        with open(DBUS_XML_PATH, encoding="utf-8") as handler:
            properties_introspection_xml: str = '\n'.join(handler.readlines())
            introspection_data = Gio.DBusNodeInfo.new_for_xml(properties_introspection_xml)
            self.reg_id = connection.register_object(self.OBJECT_PATH, introspection_data.interfaces[0], self.handle_method_call, None, None)
        assert self.reg_id > 0
        self.registered_event.set()
        logging.info("interface registered")

    def handle_method_call(self, connection: Gio.DBusConnection, sender: str, object_path: str, interface_name: str, method_name: str, parameters: GLib.Variant, invocation: Gio.DBusMethodInvocation) -> None:
        """
        Handles method calls
        """
        assert interface_name == self.INTERFACE_NAME, f"Unknown interface {interface_name}"
        logging.info("method call %s", method_name)

        if method_name == "evaluateScript":
            if not parameters.is_of_type(GLib.VariantType("(s)")):
                invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.INVALID_ARGS, f"Incorrect signature {parameters.get_type_string()}")
                return
            self.script = parameters.unpack()[0]
            self.script_set_event.set()
            invocation.return_value(GLib.Variant.new_tuple(GLib.Variant("s", "")))
        else:
            invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.UNKNOWN_METHOD, f"Unknown method {method_name}")


class InstallWallpaperTest(unittest.TestCase):
    """
    Tests for plasma-apply-wallpaperimage
    """

    loop_thread: GLibMainLoopThread
    plasmashell_interface: OrgKdePlasmashell

    @classmethod
    def setUpClass(cls) -> None:
        cls.loop_thread = GLibMainLoopThread()
        cls.loop_thread.start()
        cls.plasmashell_interface = OrgKdePlasmashell()

    def tearDown(self) -> None:
        self.plasmashell_interface.script_set_event.clear()

    @classmethod
    def tearDownClass(cls) -> None:
        cls.plasmashell_interface.quit()
        cls.loop_thread.quit()

    def test_0_install_image(self) -> None:
        """
        Install a png file
        """
        # Create an image file
        bits_per_sample = 8
        width = height = 16
        pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, bits_per_sample, width, height)
        pixbuf.fill(0xff0000ff)
        with tempfile.TemporaryDirectory() as temp_dir:
            image_path = os.path.join(temp_dir, "test.png")
            self.assertTrue(pixbuf.savev(image_path, "png"))
            subprocess.check_call([BIN_PATH, image_path])
            self.assertTrue(self.plasmashell_interface.script_set_event.wait(10))
            self.assertIn(image_path, self.plasmashell_interface.script)

    def test_1_install_kpackage(self) -> None:
        """
        Install a package
        """
        package_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, os.pardir, "wallpapers/image/plugin/autotests/testdata/default/package")
        self.assertTrue(os.path.exists(package_path))
        subprocess.check_call([BIN_PATH, package_path])
        self.assertTrue(self.plasmashell_interface.script_set_event.wait(10))
        self.assertIn(package_path, self.plasmashell_interface.script)

    def test_2_install_nonexistent_path(self) -> None:
        """
        The file does not exist
        """
        self.assertRaises(subprocess.CalledProcessError, subprocess.check_call, [BIN_PATH, "thisfiledoesnotexist.png"])
        self.assertFalse(self.plasmashell_interface.script_set_event.wait(1))

    def test_3_install_dangerous_path(self) -> None:
        """
        There is a stray single quote in the filename of this wallpaper (')
        """
        # Create an image file
        bits_per_sample = 8
        width = height = 16
        pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, bits_per_sample, width, height)
        pixbuf.fill(0xff0000ff)
        with tempfile.TemporaryDirectory() as temp_dir:
            image_path = os.path.join(temp_dir, "konqi'sfavorite.png")
            self.assertTrue(pixbuf.savev(image_path, "png"))
            self.assertRaises(subprocess.CalledProcessError, subprocess.check_call, [BIN_PATH, image_path])
            self.assertFalse(self.plasmashell_interface.script_set_event.wait(1))


if __name__ == '__main__':
    assert os.path.exists(BIN_PATH)
    logging.getLogger().setLevel(logging.INFO)
    unittest.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import unittest
from typing import Final

from appium.webdriver.common.appiumby import AppiumBy

import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from utils.base_test import KCMTest

KCM_ID: Final = "kcm_style"


class KCMStyleTest(KCMTest):
    """
    Tests for kcm_style
    """

    kcm_id = KCM_ID

    def test_0_open(self) -> None:
        """
        Tests the KCM can be opened
        """
        self.driver.find_element(AppiumBy.NAME, "Fusion")


if __name__ == '__main__':
    unittest.main()



# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

"""Notification utilities for tests."""

from typing import Any

import gi

gi.require_version('Gdk', '4.0')
from gi.repository import Gio, GLib

BUS_NAME: str = "org.freedesktop.Notifications"
OBJECT_PATH: str = "/org/freedesktop/Notifications"
IFACE_NAME: str = BUS_NAME


def send_notification(
    data: dict[str, str | int | list[str] | dict[str, GLib.Variant] | GLib.Variant],
    session_bus: Gio.DBusConnection | None = None
) -> int:
    """
    Send a notification via D-Bus.
    
    Args:
        data: Notification data dictionary with keys like app_name, summary, body, etc.
        session_bus: Optional D-Bus connection. If None, uses default session bus.
    
    Returns:
        Notification id
    """
    app_name: str = str(data.get("app_name", "Appium Test"))
    replaces_id: int = int(data.get("replaces_id", 0))
    app_icon: str = str(data.get("app_icon", "wayland"))
    summary: str = str(data.get("summary", ""))
    body: str = str(data.get("body", ""))
    actions: list[str] = data.get("actions", [])  # type: ignore
    hints: dict[str, GLib.Variant] = data.get("hints", {})  # type: ignore
    timeout: int = data.get("timeout", -1)  # type: ignore
    parameters = GLib.Variant("(susssasa{sv}i)", [app_name, replaces_id, app_icon, summary, body, actions, hints, timeout])

    if session_bus is None:
        session_bus = Gio.bus_get_sync(Gio.BusType.SESSION)

    reply = session_bus.call_sync(BUS_NAME, OBJECT_PATH, IFACE_NAME, "Notify", parameters, None, Gio.DBusSendMessageFlags.NONE, 5000)
    return reply.get_child_value(0).get_uint32()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2025 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import os
import sys

from PySide6.QtGui import QIcon
from PySide6.QtWidgets import QApplication, QSystemTrayIcon


class TestApplication(QApplication):

    def __init__(self, argv) -> None:
        super().__init__(argv)

        self.setApplicationName("SvgTrayIconTest")
        self.tray_icon = QSystemTrayIcon(QIcon(os.path.join(os.path.join(os.path.dirname(os.path.abspath(__file__))), "svgtrayicon.svg")), self)
        self.tray_icon.activated.connect(self.on_tray_icon_activated)
        self.tray_icon.show()

    def on_tray_icon_activated(self, reason) -> None:
        if reason is QSystemTrayIcon.Trigger:
            self.quit()


if __name__ == "__main__":
    app = TestApplication(sys.argv)
    sys.exit(app.exec())



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

# pylint: disable=too-many-arguments

import logging
import signal
from typing import Final

from gi.repository import Gio, GLib


class OrgKdeTestdbusactivation:
    """
    DBus interface for org.kde.testdbusactivation
    """

    BUS_NAME: Final = "org.kde.testdbusactivation"
    OBJECT_PATH: Final = "/test"

    connection: Gio.DBusConnection | None = None

    def __init__(self) -> None:
        self.reg_id: int = 0
        self.owner_id: int = Gio.bus_own_name(Gio.BusType.SESSION, self.BUS_NAME, Gio.BusNameOwnerFlags.NONE, self.on_bus_acquired, None, None)
        assert self.owner_id > 0

    def quit(self) -> None:
        if self.connection is None:
            return
        self.connection.unregister_object(self.reg_id)
        Gio.bus_unown_name(self.owner_id)
        self.connection.flush_sync(None)

    def on_bus_acquired(self, connection: Gio.DBusConnection, name: str, *args) -> None:
        """
        Interface is ready, now register objects.
        """
        self.connection = connection

        introspection_data = Gio.DBusNodeInfo.new_for_xml("""
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN"
    "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node>
  <interface name="org.kde.testdbusactivation">
    <method name="ping" />
  </interface>
</node>
""")
        self.reg_id = connection.register_object(self.OBJECT_PATH, introspection_data.interfaces[0], self.handle_method_call, None, None)
        assert self.reg_id > 0
        print("READY", flush=True)
        logging.info("Interface is ready")

    def handle_method_call(self, connection: Gio.DBusConnection, sender: str, object_path: str, interface_name: str, method_name: str, parameters: GLib.Variant, invocation: Gio.DBusMethodInvocation) -> None:
        invocation.return_value(None)


dbus_interface: OrgKdeTestdbusactivation | None = None

if __name__ == '__main__':
    logging.getLogger().setLevel(logging.INFO)
    dbus_interface = OrgKdeTestdbusactivation()

    loop = GLib.MainLoop()

    def on_signal(sig: int, frame) -> None:
        logging.info(f"Received signal {sig}, quitting...")
        if dbus_interface:
            dbus_interface.quit()
        loop.quit()

    signal.signal(signal.SIGTERM, on_signal)
    signal.signal(signal.SIGINT, on_signal)

    loop.run()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import base64
import os
import shutil
import tempfile
import unittest
from typing import Final

import gi
from appium.webdriver.common.appiumby import AppiumBy
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

gi.require_version('Gdk', '4.0')
gi.require_version('GdkPixbuf', '2.0')
from gi.repository import Gdk, GdkPixbuf, GLib

import sys

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from utils.base_test import KCMTest

KCM_ID: Final = "kcm_cursortheme"


class KCMCursorThemeTest(KCMTest):
    """
    Tests for kcm_cursortheme
    """

    kcm_id = KCM_ID
    extra_environ = {
        "QT_LOGGING_RULES": "kcm_cursortheme.debug=true",
    }

    @classmethod
    def setUpClass(cls) -> None:
        user_data_dir: Final[str] = GLib.get_user_data_dir()
        icons_folder: Final = os.path.join(user_data_dir, "icons")
        if not os.path.exists(icons_folder):
            os.mkdir(icons_folder)
            cls.addClassCleanup(lambda: shutil.rmtree(icons_folder))

        icon_theme_folder: Final = os.path.join(user_data_dir, "icons", "testcursortheme")
        assert not os.path.exists(icon_theme_folder)
        test_data_folder: Final = os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir, os.pardir, "kcms", "cursortheme", "autotests", "data", "testcursortheme")
        assert os.path.exists(test_data_folder)
        shutil.copytree(test_data_folder, icon_theme_folder)
        cls.addClassCleanup(lambda: shutil.rmtree(icon_theme_folder))

        cls.extra_environ = {
            "QT_LOGGING_RULES": "kcm_cursortheme.debug=true",
            "XCURSOR_PATH": icons_folder,
        }
        super().setUpClass()

    def test_0_open(self) -> None:
        """
        Tests the KCM and the launcher feedback dialog can be opened
        """
        self.driver.find_element(AppiumBy.NAME, "&Configure Launch Feedback…")
        self.driver.find_element(AppiumBy.NAME, "Test Cursor Theme (DO NOT TRANSLATE)")

        self.driver.find_element(AppiumBy.NAME, "&Configure Launch Feedback…").click()
        self.driver.find_element(AppiumBy.NAME, "Blinking")
        close_button = self.driver.find_element(AppiumBy.XPATH, "//button[@name='Close' and contains(@accessibility-id, 'LaunchFeedbackDialog')]")
        close_button.click()
        WebDriverWait(self.driver, 10).until(EC.invisibility_of_element(close_button))

    def test_1_cursor_theme_preview(self) -> None:
        """
        Tests if the cursor preview is loaded
        """
        with tempfile.TemporaryDirectory() as temp_dir:
            saved_image_path: str = os.path.join(temp_dir, "kcm_window.png")
            self.driver.get_screenshot_as_file(saved_image_path)
            full_image = base64.b64encode(Gdk.Texture.new_from_filename(saved_image_path).save_to_png_bytes().get_data()).decode()

        # Red
        partial_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 16, 16)
        partial_pixbuf.fill(0xff0000ff)
        partial_image = base64.b64encode(Gdk.Texture.new_for_pixbuf(partial_pixbuf).save_to_png_bytes().get_data()).decode()
        self.driver.find_image_occurrence(full_image, partial_image)


if __name__ == '__main__':
    unittest.main(failfast=True)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import os
import subprocess
import sys
import unittest
from typing import Final

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

WIDGET_ID: Final = "org.kde.plasma.cameraindicator"


class CameraIndicatorTest(unittest.TestCase):
    """
    Tests for the camera indicator widget
    """

    driver: webdriver.Remote
    pipewire_already_running_before_test: bool = False
    process_list: list[subprocess.Popen] = []

    @classmethod
    def setUpClass(cls) -> None:
        """
        Opens the widget and initialize the webdriver
        """
        options = AppiumOptions()
        options.set_capability("app", f"plasmawindowed -p org.kde.plasma.nano {WIDGET_ID}")
        options.set_capability("environ", {
            "LC_ALL": "en_US.UTF-8",
            "QT_LOGGING_RULES": "qt.accessibility.atspi.warning=false;qt.qml.propertyCache.append.warning=false",
        })
        options.set_capability("timeouts", {'implicit': 10000})
        cls.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)

        cls.pipewire_already_running_before_test = subprocess.Popen(["pidof", "pipewire"]).wait() == 0

    def tearDown(self) -> None:
        """
        Take screenshot when the current test fails
        """
        if not self._outcome.result.wasSuccessful():
            self.driver.get_screenshot_as_file(f"failed_test_shot_cameraindicatortest_#{self.id()}.png")

    @classmethod
    def tearDownClass(cls) -> None:
        """
        Make sure to terminate the driver again, lest it dangles.
        """
        assert len(cls.process_list) > 0, len(cls.process_list)
        for p in reversed(cls.process_list):
            p.kill()
            p.wait(10)
        cls.driver.quit()

    def test_0_open(self) -> None:
        """
        Tests the widget can be opened
        """
        if self.pipewire_already_running_before_test:
            self.driver.find_element(AppiumBy.NAME, "No camera is in use")
        else:
            self.driver.find_element(AppiumBy.NAME, "Camera indicator is unavailable")

    def test_1_connect_to_pipewire(self) -> None:
        """
        Tests the widget can connect to pipewire
        """
        if self.pipewire_already_running_before_test:
            self.skipTest("Pipewire is already running.")

        self.process_list.append(subprocess.Popen(["pipewire"], stdout=sys.stderr, stderr=sys.stderr))
        self.process_list.append(subprocess.Popen(["wireplumber"], stdout=sys.stderr, stderr=sys.stderr))

        # Reconnecting takes at least 5s
        WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((AppiumBy.NAME, "No camera is in use")))

    def test_2_camera_stream(self) -> None:
        """
        Camera status is reported in the widget
        """
        # Open two mock cameras to show the camera list
        bin_dir = os.getenv("CMAKE_RUNTIME_OUTPUT_DIRECTORY", os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, os.pardir, "build", "bin"))
        mock_camera1 = subprocess.Popen([os.path.join(bin_dir, "pipewiremockcamera")], stdout=subprocess.PIPE, stderr=sys.stderr)
        mock_camera2 = subprocess.Popen([os.path.join(bin_dir, "pipewiremockcamera")], stdout=sys.stderr, stderr=sys.stderr)
        self.process_list.append(mock_camera1)
        self.process_list.append(mock_camera2)
        self.driver.find_element(AppiumBy.NAME, "KDE Camera")
        self.driver.find_element("description", "Suspended")

        # Get the node id of the first mock camera
        node_id: str = ""
        PREFIX: Final = "node id: "
        while True:
            line = mock_camera1.stdout.readline().decode().strip()
            if line.startswith(PREFIX):
                node_id = line[len(PREFIX):]
                break
        assert int(node_id) > 0, node_id

        # Open the camera player and connect to the first mock camera
        camera_player = subprocess.Popen([os.path.join(bin_dir, "pipewirecameraplay"), node_id], stdout=sys.stderr, stderr=sys.stderr)
        self.process_list.append(camera_player)
        self.driver.find_element("description", "Active")

        # Show the placeholder message when there is only one camera
        mock_camera2.terminate()
        self.driver.find_element(AppiumBy.NAME, "A camera is in use")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import unittest
from typing import Final

from appium.webdriver.common.appiumby import AppiumBy

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from utils.base_test import KCMTest

KCM_ID: Final = "kcm_users"


class KCMUsersTest(KCMTest):
    """
    Tests for kcm_users
    """

    kcm_id = KCM_ID

    def test_0_open(self) -> None:
        """
        Tests the KCM can be opened
        """
        self.driver.find_element(AppiumBy.NAME, "Add New User…")
        self.driver.find_element(AppiumBy.NAME, "Name:")
        self.driver.find_element(AppiumBy.NAME, "Change Password")
        self.driver.find_element(AppiumBy.NAME, "Delete User…")

    def test_1_pictures_sheet_1_open(self) -> None:
        """
        Opens the avatar sheet
        """
        self.driver.find_element(AppiumBy.NAME, "Change avatar").click()
        self.driver.find_element(AppiumBy.NAME, "Choose File…")
        #  Doesn't work without account service
        # self.driver.find_element(AppiumBy.NAME, "Initials")
        # self.driver.find_element(AppiumBy.NAME, "Placeholder Icon")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# SPDX-License-Identifier: BSD-3-Clause
# SPDX-FileCopyrightText: 2022-2023 Harald Sitter <sitter@kde.org>

import logging
import os
import shutil
import subprocess
import sys
import tempfile
import time
import unittest
from typing import Final

from gi.repository import Gio, GLib

sys.path.append(os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, os.pardir, "utils"))
from GLibMainLoopThread import GLibMainLoopThread

KDE_VERSION: Final = 6
KACTIVITYMANAGERD_SERVICE_NAME: Final = "org.kde.ActivityManager"
KACTIVITYMANAGERD_PATH: Final = os.getenv("KACTIVITYMANAGERD_PATH", "/usr/libexec/kactivitymanagerd")
QMLTEST_EXEC: Final = os.getenv("QMLTEST_EXEC", "/usr/bin/qmltestrunner6")


def name_has_owner(session_bus: Gio.DBusConnection, name: str) -> bool:
    """
    Whether the given name is available on session bus
    """
    message: Gio.DBusMessage = Gio.DBusMessage.new_method_call("org.freedesktop.DBus", "/", "org.freedesktop.DBus", "NameHasOwner")
    message.set_body(GLib.Variant("(s)", [name]))
    reply, _ = session_bus.send_message_with_reply_sync(message, Gio.DBusSendMessageFlags.NONE, 1000)
    return reply and reply.get_signature() == 'b' and reply.get_body().get_child_value(0).get_boolean()


def build_ksycoca() -> None:
    subprocess.check_call([f"kbuildsycoca{KDE_VERSION}"], stdout=sys.stderr, stderr=sys.stderr, env=os.environ)


def start_kactivitymanagerd() -> subprocess.Popen:
    session_bus: Gio.DBusConnection = Gio.bus_get_sync(Gio.BusType.SESSION)
    assert not name_has_owner(session_bus, KACTIVITYMANAGERD_SERVICE_NAME)

    os.makedirs(os.path.join(GLib.get_user_config_dir(), "menus"))
    shutil.copy(os.path.join(os.path.dirname(os.path.abspath(__file__)), "applications.menu"), os.path.join(GLib.get_user_config_dir(), "menus"))

    kactivitymanagerd = subprocess.Popen([KACTIVITYMANAGERD_PATH], stdout=sys.stderr, stderr=sys.stderr, env=os.environ)
    kactivitymanagerd_started: bool = False
    for _ in range(10):
        if name_has_owner(session_bus, KACTIVITYMANAGERD_SERVICE_NAME):
            kactivitymanagerd_started = True
            break
        logging.info("waiting for kactivitymanagerd to appear on the DBus session")
        time.sleep(1)
    assert kactivitymanagerd_started

    build_ksycoca()

    return kactivitymanagerd


class TestDBusInterface:
    """
    D-Bus interface for org.kde.kickertest
    """

    BUS_NAME: Final = "org.kde.kickertest"
    OBJECT_PATH: Final = "/test"
    INTERFACE_NAME: Final = "org.kde.kickertest"

    connection: Gio.DBusConnection

    def __init__(self) -> None:
        self.reg_id: int = 0
        self.owner_id: int = Gio.bus_own_name(Gio.BusType.SESSION, self.BUS_NAME, Gio.BusNameOwnerFlags.NONE, self.on_bus_acquired, None, None)
        assert self.owner_id > 0

    def on_bus_acquired(self, connection: Gio.DBusConnection, name: str, *args) -> None:
        """
        The interface is ready, now register objects.
        """
        self.connection = connection
        introspection_data = Gio.DBusNodeInfo.new_for_xml("""
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN"
    "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node>
  <interface name="org.kde.kickertest">
    <method name="DeleteAndRebuildDatabase1"/>
    <method name="DeleteAndRebuildDatabase2"/>
  </interface>
</node>
""")
        self.reg_id = connection.register_object(self.OBJECT_PATH, introspection_data.interfaces[0], self.handle_method_call, None, None)
        assert self.reg_id > 0
        logging.info("interface registered")

    def handle_method_call(self, connection: Gio.DBusConnection, sender: str, object_path: str, interface_name: str, method_name: str, parameters: GLib.Variant, invocation: Gio.DBusMethodInvocation) -> None:
        logging.info("method call %s", method_name)

        if method_name == "DeleteAndRebuildDatabase1":
            os.remove(KickerTest.desktop_entry_1)
            build_ksycoca()
            invocation.return_value(None)
        elif method_name == "DeleteAndRebuildDatabase2":
            os.remove(KickerTest.desktop_entry_2)
            build_ksycoca()
            invocation.return_value(None)


class KickerTest(unittest.TestCase):
    kactivitymanagerd: subprocess.Popen
    loop_thread: GLibMainLoopThread
    dbus_interface: TestDBusInterface

    temp_dir: tempfile.TemporaryDirectory
    desktop_entry_1: str
    desktop_entry_2: str

    @classmethod
    def setUpClass(cls) -> None:
        # Prepare desktop files
        # 1
        os.makedirs(os.path.join(GLib.get_user_data_dir(), "applications"))
        shutil.copy(os.path.join(os.path.dirname(os.path.abspath(__file__)), "kickertest.desktop"), os.path.join(GLib.get_user_data_dir(), "applications"))
        cls.desktop_entry_1 = os.path.join(GLib.get_user_data_dir(), "applications", "kickertest.desktop")
        # 2
        cls.temp_dir = tempfile.TemporaryDirectory()
        os.makedirs(os.path.join(cls.temp_dir.name, "applications"))
        shutil.copy(os.path.join(os.path.dirname(os.path.abspath(__file__)), "kickertest.desktop"), os.path.join(cls.temp_dir.name, "applications"))
        cls.desktop_entry_2 = os.path.join(cls.temp_dir.name, "applications", "kickertest.desktop")

        os.environ["LC_ALL"] = "en_US.UTF-8"
        os.environ["QT_LOGGING_RULES"] = "org.kde.plasma.kicker.debug=true;kf.coreaddons.kdirwatch.debug=true"
        os.environ["XDG_DATA_DIRS"] = os.environ["XDG_DATA_DIRS"] + ":" + cls.temp_dir.name

        cls.kactivitymanagerd = start_kactivitymanagerd()

        cls.loop_thread = GLibMainLoopThread()
        cls.loop_thread.start()
        cls.dbus_interface = TestDBusInterface()

    @classmethod
    def tearDownClass(cls) -> None:
        cls.loop_thread.quit()
        cls.kactivitymanagerd.kill()
        cls.kactivitymanagerd.wait(10)

    def test_qml(self) -> None:
        """
        1. Add an entry to Favorites
        2. Remove the entry from Favorites
        3. Hide invalid entries automatically and don't crash when there are multiple entries with the same desktop name
        """
        with subprocess.Popen([QMLTEST_EXEC, "-input", os.path.join(os.path.dirname(os.path.abspath(__file__)), 'favoritetest.qml')], stdout=sys.stderr, stderr=sys.stderr) as process:
            self.assertEqual(process.wait(60), 0)


if __name__ == '__main__':
    assert "USE_CUSTOM_BUS" in os.environ
    logging.getLogger().setLevel(logging.INFO)
    unittest.main()



# SPDX-FileCopyrightText: 2026 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT
"""Process management utilities for tests."""

import subprocess
import time

from .constants import KDE_VERSION, PROCESS_WAIT_INTERVAL, PROCESS_WAIT_ITERATIONS


def quit_plasmawindowed() -> None:
    """Quit plasmawindowed process."""
    subprocess.check_call([f"kquitapp{KDE_VERSION}", "plasmawindowed"])


def wait_for_process_exit(process_name: str) -> None:
    """Wait for a process to exit."""
    for _ in range(PROCESS_WAIT_ITERATIONS):
        try:
            subprocess.check_call(["pidof", process_name])
        except subprocess.CalledProcessError:
            break
        time.sleep(PROCESS_WAIT_INTERVAL)


def cleanup_plasmawindowed() -> None:
    """Clean up plasmawindowed process."""
    quit_plasmawindowed()
    wait_for_process_exit("plasmawindowed")



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import os
import subprocess
import time
import unittest

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy


class LockedTest(unittest.TestCase):

    driver: webdriver.Remote

    @classmethod
    def setUpClass(cls) -> None:
        options = AppiumOptions()
        options.set_capability("app", f"{os.getenv('QML_EXEC')} {os.path.dirname(os.path.realpath(__file__))}/lockedtest.qml")
        cls.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)
        options.set_capability("timeouts", {'implicit': 10000})

    @classmethod
    def tearDownClass(cls) -> None:
        # Make sure to terminate the driver again, lest it dangles.
        cls.driver.quit()

    def test_caps_lock(self) -> None:
        self.driver.find_element(AppiumBy.NAME, "unlocked")
        subprocess.check_call(["xdotool", "key", "Caps_Lock"])
        self.driver.find_element(AppiumBy.NAME, "locked")
        subprocess.check_call(["xdotool", "key", "Caps_Lock"])
        self.driver.find_element(AppiumBy.NAME, "unlocked")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
SPDX-FileCopyrightText: 2025 David Edmundson <davidedmundson@kde.org>
SPDX-FileCopyrightText: 2026 Fushan Wen <qydwhotmail@gmail.com>
SPDX-License-Identifier: LGPL-2.0-or-later
"""

import sys

from KGuiAddons import KSystemClipboard
from PySide6.QtCore import QMimeData, Slot
from PySide6.QtGui import QClipboard
from PySide6.QtWidgets import (QApplication, QGroupBox, QHBoxLayout, QPushButton, QTextEdit, QVBoxLayout, QWidget)


class ClipboardTestWindow(QWidget):
    """
    This is a small GUI app which constantly syncs the normal and data control
    clipboards. This is useful for confirming all deadlocks are gone.

    @see https://invent.kde.org/frameworks/kguiaddons/-/merge_requests/191
    """

    def __init__(self) -> None:
        super().__init__()
        self.setWindowTitle("Clipboard Test")
        self.__setup_ui()

    def __setup_ui(self) -> None:
        self.root_layout = QHBoxLayout(self)
        self.normal_clipboard_view = QTextEdit()
        self.normal_text_edit = QTextEdit()
        self.__setup_regular_clipboard(self.root_layout)

        self.system_clipboard_view = QTextEdit()
        self.system_text_edit = QTextEdit()
        self.__setup_system_clipboard(self.root_layout)

        self.resize(600, 500)

    def __setup_regular_clipboard(self, parent_layout) -> None:
        clipboard = QApplication.clipboard()

        group = QGroupBox("Normal Clipboard")
        parent_layout.addWidget(group)

        group_layout = QVBoxLayout(group)

        self.normal_clipboard_view.setAccessibleName("Normal Clipboard View")
        self.normal_clipboard_view.setReadOnly(True)
        self.normal_clipboard_view.setMinimumHeight(200)

        clipboard.changed.connect(self.on_normal_clipboard_changed)

        self.normal_clipboard_view.setText(clipboard.text())

        group_layout.addWidget(self.normal_clipboard_view)

        self.normal_text_edit.setAccessibleName("Normal Text Edit")
        self.normal_text_edit.setPlaceholderText("Enter clipboard content here…")
        self.normal_text_edit.setMinimumHeight(120)
        group_layout.addWidget(self.normal_text_edit)

        btn_row = QHBoxLayout()
        btn_row.addStretch()

        submit_btn = QPushButton("Submit1")
        submit_btn.clicked.connect(self.submit_normal_clipboard)
        btn_row.addWidget(submit_btn)

        group_layout.addLayout(btn_row)

    def __setup_system_clipboard(self, parent_layout) -> None:
        clipboard = KSystemClipboard.instance()

        group = QGroupBox("System Clipboard (QClipboard)")
        parent_layout.addWidget(group)

        group_layout = QVBoxLayout(group)

        self.system_clipboard_view.setAccessibleName("System Clipboard View")
        self.system_clipboard_view.setReadOnly(True)
        self.system_clipboard_view.setMinimumHeight(200)

        clipboard.changed.connect(self.on_system_clipboard_changed)

        self.system_clipboard_view.setText(clipboard.text(QClipboard.Mode.Clipboard))

        group_layout.addWidget(self.system_clipboard_view)

        self.system_text_edit.setAccessibleName("System Text Edit")
        self.system_text_edit.setPlaceholderText("Enter clipboard content here…")
        self.system_text_edit.setMinimumHeight(120)
        group_layout.addWidget(self.system_text_edit)

        btn_row = QHBoxLayout()
        btn_row.addStretch()

        submit_btn = QPushButton("Submit2")
        submit_btn.clicked.connect(self.submit_system_clipboard)
        btn_row.addWidget(submit_btn)

        group_layout.addLayout(btn_row)

    @Slot(QClipboard.Mode)
    def on_normal_clipboard_changed(self, mode) -> None:
        if mode != QClipboard.Mode.Clipboard:
            return
        clipboard = QApplication.clipboard()
        self.normal_clipboard_view.setText(clipboard.text(mode))

    @Slot(QClipboard.Mode)
    def on_system_clipboard_changed(self, mode) -> None:
        if mode != QClipboard.Mode.Clipboard:
            return
        clipboard = KSystemClipboard.instance()
        self.system_clipboard_view.setText(clipboard.text(mode))

    @Slot()
    def submit_normal_clipboard(self) -> None:
        clipboard = QApplication.clipboard()
        mime_data = QMimeData()
        mime_data.setText(self.normal_text_edit.toPlainText())
        clipboard.setMimeData(mime_data, QClipboard.Mode.Clipboard)

    @Slot()
    def submit_system_clipboard(self) -> None:
        clipboard = KSystemClipboard.instance()
        mime_data = QMimeData()
        mime_data.setText(self.system_text_edit.toPlainText())
        clipboard.setMimeData(mime_data, QClipboard.Mode.Clipboard)


if __name__ == "__main__":
    app = QApplication(sys.argv)

    window = ClipboardTestWindow()
    window.show()

    sys.exit(app.exec())



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import os
import shutil
import unittest
from typing import Final

from appium.webdriver.common.appiumby import AppiumBy
from gi.repository import GLib

import sys
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from utils.base_test import KCMTest

KCM_ID: Final = "kcm_soundtheme"


class KCMSoundThemeTest(KCMTest):
    """
    Tests for kcm_soundtheme
    """

    kcm_id = KCM_ID

    @classmethod
    def setUpClass(cls) -> None:
        system_data_dirs: Final[list[str]] = GLib.get_system_data_dirs()
        freedesktop_sound_theme_installed: bool = False
        for system_data_dir in system_data_dirs:
            if os.path.exists(os.path.join(system_data_dir, "sounds", "freedesktop", "index.theme")):
                freedesktop_sound_theme_installed = True
                break

        if not freedesktop_sound_theme_installed:
            user_data_dir: Final[str] = GLib.get_user_data_dir()
            sounds_folder: Final = os.path.join(user_data_dir, "sounds")
            if not os.path.exists(sounds_folder):
                os.mkdir(sounds_folder)
                cls.addClassCleanup(lambda: shutil.rmtree(sounds_folder))

            fdo_sound_theme_folder: Final = os.path.join(user_data_dir, "sounds", "freedesktop")
            if not os.path.exists(fdo_sound_theme_folder):
                shutil.copytree("../../kcms/soundtheme/autotests/data/freedesktop", fdo_sound_theme_folder)
                cls.addClassCleanup(lambda: shutil.rmtree(fdo_sound_theme_folder))

        super().setUpClass()

    def test_0_open(self) -> None:
        """
        Tests the KCM can be opened
        """
        self.driver.find_element(AppiumBy.NAME, "FreeDesktop")
        self.driver.find_element(AppiumBy.NAME, "Fallback sound theme from freedesktop.org")
        self.driver.find_element(AppiumBy.NAME, "Preview:")


if __name__ == '__main__':
    unittest.main()



#
# SPDX-FileCopyrightText: 2024 Niccolò Venerandi <niccolo@venerandi.com>
#
# SPDX-License-Identifier: GPL-2.0-or-later
#

import requests
import subprocess
import zipfile
import os

def download_latest_release_file(owner, repo, file_name, download_dir="."):
    # GitHub API URL for latest release
    api_url = f"https://api.github.com/repos/{owner}/{repo}/releases/latest"

    try:
        # Get the latest release info
        response = requests.get(api_url)
        response.raise_for_status()
        release_data = response.json()

        # Find the asset with the specified file name
        for asset in release_data["assets"]:
            if asset["name"] == file_name:
                download_url = asset["browser_download_url"]
                break
        else:
            print(f"File '{file_name}' not found in the latest release.")
            return

        # Download the file
        response = requests.get(download_url, stream=True)
        response.raise_for_status()

        # Save the file
        os.makedirs(download_dir, exist_ok=True)
        file_path = os.path.join(download_dir, file_name)
        with open(file_path, "wb") as file:
            for chunk in response.iter_content(chunk_size=8192):
                file.write(chunk)

        print(f"Downloaded '{file_name}' to '{file_path}'")

    except requests.exceptions.RequestException as e:
        print(f"Error downloading file: {e}")

def check_command(command):
    """Check if a command exists in the system."""
    return subprocess.call(['which', command], stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0

def unzip_file(zip_file_path, extract_to='.'):
    """Unzip the downloaded file."""
    print(f"Unzipping {zip_file_path}...")
    with zipfile.ZipFile(zip_file_path, 'r') as zip_ref:
        zip_ref.extractall(extract_to)
    print(f"Extracted to {extract_to}")

def run_mapshaper(json_file, output_name):
    """Run mapshaper on the specified JSON file."""
    print(f"Running mapshaper on {json_file}...")
    subprocess.run(['mapshaper', json_file, "-simplify", "0.2%", "keep-shapes", "-o", output_name])

# Example usage
owner = "evansiroky"  # GitHub repository owner
repo = "timezone-boundary-builder"    # GitHub repository name
file_name = "timezones.geojson.zip"  # Replace with the name of the file you want to download
download_dir = "."    # Directory to save the file
json_file = "combined.json"
output_name = "timezones.json"


if not check_command('mapshaper'):
    print('The "mapshaper" command line utility is required to update the timezone information. See https://github.com/mbloch/mapshaper for more information')

download_latest_release_file(owner, repo, file_name, download_dir)

unzip_file(file_name)

run_mapshaper(json_file, output_name)

os.remove(file_name)

os.remove(json_file)





#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: GPL-2.0-or-later

# For FreeBSD CI which only has python 3.9
from __future__ import annotations

import threading
import time

from gi.repository import GLib


class GLibMainLoopThread(threading.Thread):
    """
    A GLib main loop in another thread.
    @note It's redundant to create a loop when QCoreApplication is used in the same application.
    """

    loop: GLib.MainLoop | None = None

    def __init__(self) -> None:
        # Set up D-Bus loop
        self.loop = GLib.MainLoop()
        self.failSafeTimer = threading.Timer(60, self.loop.quit)

        # Create the thread
        super(GLibMainLoopThread, self).__init__()

    def run(self) -> None:
        """
        Method to run the DBus main loop (on a thread)
        """
        self.failSafeTimer.start()
        self.loop.run()

    def quit(self) -> None:
        self.failSafeTimer.cancel()
        self.loop.quit()

    @classmethod
    def process_events(cls) -> None:
        """
        Processes some pending events in the main loop
        """
        if cls.loop is not None:
            context = cls.loop.get_context()
        else:
            context = GLib.MainContext.default()

        for _ in range(4):
            if not context.pending():
                time.sleep(0.1)
                continue
            context.iteration(may_block=False)
            break



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2025 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

from typing import Final

from gi.repository import Gio, GLib


class JobNotificationControlInterface:

    BUS_NAME: Final = "org.kde.jobnotificationtest"
    OBJECT_PATH: Final = "/jobnotificationtest"

    connection: Gio.DBusConnection
    pid: int

    def __init__(self, job) -> None:
        self.reg_id: int = 0
        self.owner_id: int = Gio.bus_own_name(Gio.BusType.SESSION, self.BUS_NAME, Gio.BusNameOwnerFlags.NONE, self.on_bus_acquired, None, None)
        assert self.owner_id > 0

        self.job = job

    def quit(self) -> None:
        self.connection.unregister_object(self.reg_id)
        Gio.bus_unown_name(self.owner_id)
        self.connection.flush_sync(None)

    def on_bus_acquired(self, connection: Gio.DBusConnection, name: str, *args) -> None:
        self.connection = connection

        introspection_data = Gio.DBusNodeInfo.new_for_xml("""
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN"
    "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node>
  <interface name="org.kde.jobnotificationtest">
    <method name="SetProcessedAmount">
      <arg name="ProcessedAmount" type="t" direction="in"/>
    </method>
  </interface>
</node>
""")
        self.reg_id = connection.register_object(self.OBJECT_PATH, introspection_data.interfaces[0], self.handle_method_call, None, None)
        assert self.reg_id > 0

    def handle_method_call(self, connection: Gio.DBusConnection, sender: str, object_path: str, interface_name: str, method_name: str, parameters: GLib.Variant, invocation: Gio.DBusMethodInvocation) -> None:
        if method_name == "SetProcessedAmount":
            self.job.update_progress(parameters[0])
        else:
            invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.UNKNOWN_METHOD, f"Unknown method {method_name}")
            return

        invocation.return_value(None)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2026 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import os
import subprocess
import sys
import unittest

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
from selenium.webdriver.support.ui import WebDriverWait


class ClipboardDeadlockTest(unittest.TestCase):
    """
    A test for the clipboard deadlock issue
    """

    driver: webdriver.Remote

    @classmethod
    def setUpClass(cls) -> None:
        options = AppiumOptions()
        options.set_capability("app", f"python3 {os.path.join(os.path.dirname(os.path.abspath(__file__)), 'clipboardtest', 'deadlocktestwindow.py')}")
        options.set_capability("environ", {
            "QT_QPA_PLATFORM": "wayland" if os.getenv("TEST_WITH_KWIN_WAYLAND", "1") != "0" else "xcb",
            "LD_PRELOAD": subprocess.check_output(["gcc", "-print-file-name=libasan.so"]).strip().decode(encoding="utf-8"),
        })
        options.set_capability("timeouts", {'implicit': 10000})
        cls.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)

    def tearDown(self) -> None:
        """
        Take screenshot when the current test fails
        """
        if not self._outcome.result.wasSuccessful():
            self.driver.get_screenshot_as_file(f"failed_test_shot_deadlocktestwindow_#{self.id()}.png")

    @classmethod
    def tearDownClass(cls) -> None:
        """
        Make sure to terminate the driver again, lest it dangles.
        """
        cls.driver.quit()

    def test_0_set_regular_clipboard(self) -> None:
        text = "From Normal Clipboard"
        self.driver.find_element(AppiumBy.NAME, "Normal Text Edit").send_keys(text)
        self.driver.find_element(AppiumBy.NAME, "Submit1").click()
        WebDriverWait(self.driver, 10).until(lambda d: d.find_element(AppiumBy.NAME, "Normal Clipboard View").text == text)
        WebDriverWait(self.driver, 10).until(lambda d: d.find_element(AppiumBy.NAME, "System Clipboard View").text == text)

    def test_1_set_system_clipboard(self) -> None:
        text = "From System Clipboard"
        self.driver.find_element(AppiumBy.NAME, "System Text Edit").send_keys(text)
        self.driver.find_element(AppiumBy.NAME, "Submit2").click()
        WebDriverWait(self.driver, 10).until(lambda d: d.find_element(AppiumBy.NAME, "Normal Clipboard View").text == text)
        WebDriverWait(self.driver, 10).until(lambda d: d.find_element(AppiumBy.NAME, "System Clipboard View").text == text)


if __name__ == '__main__':
    try:
        from PySide6.QtCore import QObject
    except ModuleNotFoundError:
        sys.exit(0)
    unittest.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import os
import subprocess
import sys
from typing import Final

CMAKE_BINARY_DIR: Final = os.getenv("CMAKE_BINARY_DIR")
QMLTEST_PATH: Final = os.getenv("QMLTEST_PATH")

if __name__ == '__main__':
    assert QMLTEST_PATH is not None
    assert os.path.exists(QMLTEST_PATH)
    result: int = 1

    with subprocess.Popen([os.path.join(os.path.dirname(os.path.abspath(__file__)), "dbusservice.py")], stdout=sys.stdout, stderr=sys.stderr) as dbus_service:
        args = [
            QMLTEST_PATH,
            "-input",
            os.path.join(os.path.dirname(os.path.abspath(__file__)), sys.argv[1]),
        ]
        if CMAKE_BINARY_DIR is not None:
            args += ["-import", CMAKE_BINARY_DIR]

        print("Starting test")
        test_runner = subprocess.Popen(args, stdout=sys.stdout, stderr=sys.stderr)
        try:
            result = test_runner.wait(timeout=60)
        except subprocess.TimeoutExpired as e:
            print("Timeout", e)
        finally:
            test_runner.terminate()
            dbus_service.terminate()

    sys.exit(result)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import os

import gi

gi.require_version('GdkPixbuf', '2.0')
gi.require_version('Gtk', '3.0')
from gi.repository import GdkPixbuf, GLib, Gtk

win = None
current_index = 0
icon_1 = GdkPixbuf.Pixbuf.new_from_file(os.path.join(os.path.dirname(os.path.abspath(__file__)), "none.png"))
icon_2 = GdkPixbuf.Pixbuf.new_from_file(os.path.join(os.path.dirname(os.path.abspath(__file__)), "samplewidgetwindow.png"))


def change_icon():
    global current_index
    if current_index == 0:
        win.set_icon(icon_1)
        current_index = 1
    else:
        win.set_icon(icon_2)
        current_index = 0

    return True


if __name__ == "__main__":
    win = Gtk.Window(title="flash")
    win.show_all()
    GLib.timeout_add(100, change_icon)
    GLib.timeout_add_seconds(60, Gtk.main_quit)
    Gtk.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import base64
import logging
import os
import shutil
import sqlite3
import subprocess
import sys
import tempfile
import threading
import time
import unittest
from typing import Final

import gi
from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
from selenium.common.exceptions import (NoSuchElementException, WebDriverException)
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.actions.action_builder import ActionBuilder
from selenium.webdriver.common.actions.interaction import POINTER_MOUSE
from selenium.webdriver.common.actions.mouse_button import MouseButton
from selenium.webdriver.common.actions.pointer_input import PointerInput
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait

gi.require_version('Gdk', '4.0')
gi.require_version('GdkPixbuf', '2.0')
gi.require_version('Gtk', '4.0')
from gi.repository import Gdk, GdkPixbuf, Gio, GLib, Gtk

WIDGET_ID: Final = "org.kde.plasma.clipboard"
KDE_VERSION: Final = 6


class TestApplication(Gtk.Application):

    def __init__(self) -> None:
        super().__init__(application_id="org.kde.plasma.clipboardtest")

        # Create history file to suppress warnings
        klipper_folder = os.path.join(GLib.get_user_data_dir(), "klipper")
        self.klipper_data_file = os.path.join(klipper_folder, "history3.sqlite")
        assert not os.path.exists(self.klipper_data_file)
        shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, os.pardir, "klipper/autotests/data/onetextentry"), klipper_folder)

        self.driver: webdriver.Remote
        self.driver_ready_event = threading.Event()
        self.connect("activate", self.on_activate)

        self.klipper_proxy = Gio.DBusProxy.new_for_bus_sync(Gio.BusType.SESSION, 0, None, "org.kde.klipper", "/klipper", "org.kde.klipper.klipper")
        self.klipper_updated_event = threading.Event()
        self.klipper_proxy.connect("g-signal", self.klipper_signal_handler)

        GLib.timeout_add_seconds(180, self.quit)

    def on_activate(self, _app: Gtk.Application) -> None:
        options = AppiumOptions()
        options.set_capability("app", f"plasmawindowed -p org.kde.plasma.nano {WIDGET_ID}")
        options.set_capability(
            "environ",
            {
                "LC_ALL": "en_US.UTF-8",
                "QT_LOGGING_RULES": "qt.accessibility.atspi.warning=false;org.kde.klipper.debug=true;qt.qml.propertyCache.append.warning=false",
            })
        options.set_capability("timeouts", {'implicit': 5000})
        self.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)
        self.driver_ready_event.set()

    def klipper_signal_handler(self, d_bus_proxy: Gio.DBusProxy, sender_name: str, signal_name: str, parameters: GLib.Variant) -> None:
        logging.info(f"received signal {signal_name}")
        if signal_name == "clipboardHistoryUpdated":
            self.klipper_updated_event.set()

    @staticmethod
    def spin() -> None:
        context = GLib.MainContext.default()
        count = 0
        while context.pending() or count < 10:
            count += 1
            if not context.pending():
                time.sleep(0.1)
                continue
            context.iteration(may_block=True)

    @staticmethod
    def spin_until(event: threading.Event) -> None:
        context = GLib.MainContext.default()
        while not event.is_set() or context.pending():
            if not context.pending():
                time.sleep(0.1)
                continue
            context.iteration(may_block=True)

    def gtk_copy(self, content_provider: Gdk.ContentProvider, clipboard_mode: int = 0) -> None:
        logging.info("gtk_copy")
        window = Gtk.Window()
        window.set_default_size(20, 20)
        button = Gtk.Button(label="Copy Content")
        window.set_child(button)
        window.set_visible(True)
        self.spin()

        # Click the button to update the latest serial. See also:
        # https://invent.kde.org/plasma/kwin/-/commit/31018c000bbad5dc3b263b7f452b0795dd153ceb
        # https://github.com/GNOME/gtk/blob/7da4844dcc2fb2a35457fc4e251c504c8f3d0206/gdk/wayland/gdkseat-wayland.c#L4390
        ActionChains(self.driver).send_keys(Keys.SPACE).perform()
        self.spin()
        if clipboard_mode == 0:
            clipboard = window.get_display().get_clipboard()  # Clipboard
        else:
            clipboard = window.get_display().get_primary_clipboard()  # Selection
        assert clipboard.set_content(content_provider)
        self.spin()

        window.set_visible(False)
        self.spin()

    def gtk_get_clipboard_mime_data(self, clipboard_mode: int = 0) -> dict[str, GLib.Bytes]:
        logging.info("gtk_get_clipboard_mime_data")
        window = Gtk.Window()
        window.set_default_size(20, 20)
        button = Gtk.Button(label="Get Content")
        window.set_child(button)
        window.set_visible(True)
        self.spin()

        if clipboard_mode == 0:
            clipboard = window.get_display().get_clipboard()  # Clipboard
        else:
            clipboard = window.get_display().get_primary_clipboard()  # Selection
        self.spin()

        data: dict[str, GLib.Bytes] = {}
        read_event = threading.Event()

        def on_value_read(_clipboard: Gdk.Clipboard, result: Gio.AsyncResult, user_data) -> None:
            nonlocal data, read_event
            stream, t = _clipboard.read_finish(result)
            logging.info(f"reading data for {t}")
            data[t] = stream.read_bytes(10000, None)
            logging.info(f"reading done")
            read_event.set()

        for t in clipboard.get_formats().get_mime_types():
            if t.startswith("image/") and not t.endswith("png"):
                continue
            if t.startswith("application/x-qt-image"):
                continue
            ActionChains(self.driver).send_keys(Keys.SPACE).perform()
            clipboard.read_async([t, None], GLib.PRIORITY_DEFAULT, None, on_value_read, None)
            self.spin_until(read_event)
            read_event.clear()

        window.set_visible(False)
        self.spin()

        return data


class ClipboardTest(unittest.TestCase):
    """
    Tests for the clipboard widget
    """

    @classmethod
    def setUpClass(cls) -> None:
        app.driver_ready_event.wait(10)

    def tearDown(self) -> None:
        """
        Take screenshot when the current test fails
        """
        if not self._outcome.result.wasSuccessful():
            app.driver.get_screenshot_as_file(f"failed_test_shot_{WIDGET_ID}_#{self.id()}.png")

    @classmethod
    def tearDownClass(cls) -> None:
        """
        Make sure to terminate the driver again, lest it dangles.
        """
        subprocess.check_call([f"kquitapp{KDE_VERSION}", "plasmawindowed"])
        for _ in range(10):
            try:
                subprocess.check_call(["pidof", "plasmawindowed"])
            except subprocess.CalledProcessError:
                break
            time.sleep(1)
        app.driver.quit()
        app.quit()

    def test_0_open(self) -> None:
        """
        Tests the widget can be opened and the history can be loaded
        """
        app.driver.find_element(AppiumBy.NAME, "Fushan Wen")
        app.driver.find_element(AppiumBy.NAME, "clipboard")

        self.assertEqual(app.klipper_proxy.getClipboardContents(), "Fushan Wen")

    def test_basic(self) -> None:
        """
        The basic part tests the GUI buttons
        """
        # Tests the barcode page can be opened
        actions = ActionChains(app.driver)
        actions.send_keys(Keys.DOWN).perform()
        # Wait until the first item is selected
        try:
            app.driver.find_element(AppiumBy.NAME, "Show QR code").click()
        except NoSuchElementException:
            actions.send_keys(Keys.DOWN).perform()  # Try pressing down key again
            app.driver.find_element(AppiumBy.NAME, "Show QR code").click()

        app.driver.find_element(AppiumBy.NAME, "QR Code")
        app.driver.find_element(AppiumBy.NAME, "Return to Clipboard")

        # Go back to the list from the barcode page
        button_item = app.driver.find_element(AppiumBy.NAME, "Return to Clipboard")
        self.assertTrue(button_item.is_displayed())
        button_item.click()
        app.driver.find_element(AppiumBy.NAME, "Fushan Wen")
        self.assertFalse(button_item.is_displayed())

        # In edit mode, the text area should be focused by default.
        ActionChains(app.driver).send_keys(Keys.DOWN).send_keys(Keys.DOWN).perform()
        app.driver.find_element(AppiumBy.NAME, "Edit contents").click()
        app.driver.find_element(AppiumBy.NAME, "Text edit area")
        time.sleep(1)
        # By default the text area is focused, so typing anything will appear in the text area.
        new_text = "clip bold"
        ActionChains(app.driver).key_down(Keys.CONTROL).send_keys("a").key_up(Keys.CONTROL).perform()  # Select all
        ActionChains(app.driver).key_down(Keys.CONTROL).send_keys("a").key_up(Keys.CONTROL).perform()  # Perform twice to make it less flaky
        ActionChains(app.driver).send_keys(new_text).pause(1).perform()
        ActionChains(app.driver).key_down(Keys.CONTROL).send_keys("s").key_up(Keys.CONTROL).perform()  # Save
        app.driver.find_element(AppiumBy.NAME, new_text)
        self.assertEqual(app.gtk_get_clipboard_mime_data()["text/plain;charset=utf-8"].get_data().decode("utf-8"), new_text)

        # BUG 494145: update uuid after editing so the item can be removed
        delete_button = app.driver.find_element(AppiumBy.NAME, "Remove from history")
        delete_button.click()
        WebDriverWait(app.driver, 5).until(EC.invisibility_of_element(delete_button))
        # The first item becomes the current clipboard item
        self.assertEqual(app.gtk_get_clipboard_mime_data()["text/plain;charset=utf-8"].get_data().decode("utf-8"), "clipboard")

        item = app.driver.find_element(AppiumBy.NAME, "clipboard")
        app.driver.find_element(AppiumBy.NAME, "Clear History").click()
        app.driver.find_element(AppiumBy.NAME, "Delete").click()
        WebDriverWait(app.driver, 5).until(EC.invisibility_of_element(item))

        # Star/Pin feature
        app.klipper_proxy.clearClipboardHistory()
        app.spin()
        for new_text in ("star 1", "normal 1"):
            content_text = Gdk.ContentProvider.new_for_bytes("text/plain;charset=utf-8", GLib.Bytes.new(bytes(new_text, "utf-8")))
            app.gtk_copy(content_text)
        app.driver.find_element(AppiumBy.NAME, "normal 1")
        ActionChains(app.driver).send_keys(Keys.DOWN).send_keys(Keys.DOWN).perform()
        app.driver.find_element(AppiumBy.NAME, "Star").click()
        tabbar_element = app.driver.find_element(AppiumBy.NAME, "Starred Only")
        tabbar_element.click()
        app.driver.find_element(AppiumBy.NAME, "star 1")
        time.sleep(1)  # Wait for animation
        self.assertRaises((WebDriverException, NoSuchElementException), app.driver.find_element, AppiumBy.NAME, "normal 1")
        ActionChains(app.driver).send_keys(Keys.DOWN).perform()
        app.driver.find_element(AppiumBy.NAME, "Remove Star").click()
        app.driver.find_element(AppiumBy.NAME, "normal 1")
        WebDriverWait(app.driver, 5).until(EC.invisibility_of_element(tabbar_element))

    def update_config(self, group: str | list[str], key: str | list[str], new_value: str | list[str]) -> None:
        if isinstance(group, str):
            subprocess.check_call([f"kwriteconfig{KDE_VERSION}", "--file", "klipperrc", "--group", group, "--key", key, new_value])
        else:
            for g, k, v in zip(group, key, new_value):
                subprocess.check_call([f"kwriteconfig{KDE_VERSION}", "--file", "klipperrc", "--group", g, "--key", k, v])

        app.klipper_proxy.reloadConfig()

    def take_screenshot(self) -> str:
        with tempfile.TemporaryDirectory() as temp_dir:
            saved_image_path = os.path.join(temp_dir, "tray.png")
            app.driver.get_screenshot_as_file(saved_image_path)
            return base64.b64encode(Gdk.Texture.new_from_filename(saved_image_path).save_to_png_bytes().get_data()).decode()

    def test_extended(self) -> None:
        """
        The extended part tests the inner klipper code.
        """
        # D-Bus interface for Klipper
        app.klipper_proxy.clearClipboardHistory()
        app.spin()
        app.klipper_updated_event.clear()
        # setClipboardContents with a valid string
        clipboard_content = "setFromTest"
        app.klipper_proxy.setClipboardContents("(s)", clipboard_content)
        element = app.driver.find_element(AppiumBy.NAME, clipboard_content)
        app.spin()
        self.assertTrue(app.klipper_updated_event.is_set())
        # self.assertEqual(app.gtk_get_clipboard_mime_data()["text/plain;charset=utf-8"].get_data().decode("utf-8"), clipboard_content) TODO: this doesn't work on Wayland

        # setClipboardContents with an empty string
        app.klipper_updated_event.clear()
        app.klipper_proxy.setClipboardContents("(s)", "")
        app.spin()
        self.assertFalse(app.klipper_updated_event.is_set())

        # History item
        self.assertIn(clipboard_content, app.klipper_proxy.getClipboardHistoryMenu())
        self.assertEqual(app.klipper_proxy.getClipboardHistoryItem("(i)", 0), clipboard_content)
        self.assertEqual(app.klipper_proxy.getClipboardHistoryItem("(i)", 123), "")  # Invalid index

        # clearClipboardHistory
        app.klipper_updated_event.clear()
        last_modified = os.stat(app.klipper_data_file).st_mtime
        app.klipper_proxy.clearClipboardHistory()
        app.klipper_proxy.saveClipboardHistory()
        app.spin()
        self.assertTrue(app.klipper_updated_event.is_set())
        self.assertFalse(element.is_displayed())
        self.assertNotEqual(last_modified, os.stat(app.klipper_data_file).st_mtime)

        # The PreviewImageProvider registers a custom image provider to load previews for URLs.
        new_text = "clip thin"
        pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 256, 256)
        with tempfile.TemporaryDirectory() as temp_dir:
            colors = (0xff0000ff, 0x00ff00ff, 0x0000ffff)
            for color in colors:
                pixbuf.fill(color)
                pixbuf.savev(os.path.join(temp_dir, f"{str(color)}.png"), "png")

            uri_list = '\r\n'.join([f"file://{os.path.join(temp_dir, path)}" for path in os.listdir(temp_dir)]) + '\r\n'
            content_urls = Gdk.ContentProvider.new_for_bytes("text/uri-list", GLib.Bytes.new(bytes(uri_list, "utf-8")))
            content_text = Gdk.ContentProvider.new_for_bytes("text/plain", GLib.Bytes.new(bytes(new_text, "utf-8")))
            content_union = Gdk.ContentProvider.new_union([content_text, content_urls])
            app.gtk_copy(content_union)  # URL has the highest priority

            time.sleep(1)  # Asynchronous loading
            partial_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 16, 16)
            for color in colors:
                partial_pixbuf.fill(color)
                partial_image = base64.b64encode(Gdk.Texture.new_for_pixbuf(partial_pixbuf).save_to_png_bytes().get_data()).decode()
                try:
                    app.driver.find_image_occurrence(self.take_screenshot(), partial_image)
                except WebDriverException:  # Flaky
                    time.sleep(1)
                    app.driver.find_image_occurrence(self.take_screenshot(), partial_image)

            self.assertRaises(NoSuchElementException, app.driver.find_element, AppiumBy.NAME, new_text)

        # A cell has both image data and text data, which should not be ignored when images are ignored.
        utf8_text = "你好世界"
        ascii_text = "helloworld"
        pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 256, 256)
        pixbuf.fill(0xff0000ff)
        content_image = Gdk.ContentProvider.new_for_bytes("image/png", Gdk.Texture.new_for_pixbuf(pixbuf).save_to_png_bytes())
        content_text = Gdk.ContentProvider.new_for_bytes("text/plain;charset=utf-8", GLib.Bytes.new(bytes(utf8_text, "utf-8")))
        content_asciitext = Gdk.ContentProvider.new_for_bytes("text/plain;charset=ANSI_X3.4-1968", GLib.Bytes.new(bytes(ascii_text, "ANSI_X3.4-1968")))
        content_union = Gdk.ContentProvider.new_union([content_text, content_asciitext, content_image])
        app.gtk_copy(content_union)
        app.driver.find_element(AppiumBy.NAME, utf8_text)

        # When a mimedata has "x-kde-passwordManagerHint" set to "secret", the clip should not be saved to history.
        try:
            from PySide6.QtCore import QObject
            process = subprocess.Popen(["python3", os.path.join(os.path.dirname(os.path.abspath(__file__)), "clipboardtest", "copysecret.py")], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            self.addCleanup(process.kill)
            process.stderr.readline()
            process.stderr.readline()
            ActionChains(app.driver).send_keys(Keys.SPACE).perform()
            app.driver.find_element(AppiumBy.NAME, "123456789")
            self.assertRaises(NoSuchElementException, app.driver.find_element, AppiumBy.NAME, "123456789test")
        except ModuleNotFoundError:
            pass

        # Klipper's history should preserve more MIME types
        app.klipper_proxy.clearClipboardHistory()

        pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 1, 1)  # 1x1 pixel
        pixbuf.fill(0xff0000ff)
        content_image = Gdk.ContentProvider.new_for_bytes("image/png", Gdk.Texture.new_for_pixbuf(pixbuf).save_to_png_bytes())
        utf8_text_data = GLib.Bytes.new(bytes("bug491961", "utf-8"))
        utf8_text = Gdk.ContentProvider.new_for_bytes("text/plain;charset=utf-8", utf8_text_data)
        ascii_text_data = GLib.Bytes.new(bytes("bug491961", "ANSI_X3.4-1968"))
        ascii_text = Gdk.ContentProvider.new_for_bytes("text/plain;charset=ANSI_X3.4-1968", ascii_text_data)
        temp_file = tempfile.NamedTemporaryFile(suffix=".txt")
        urls_data = GLib.Bytes.new(bytes(f"file://{temp_file.name}\r\n", "utf-8"))
        content_urls = Gdk.ContentProvider.new_for_bytes("text/uri-list", urls_data)
        content_application = Gdk.ContentProvider.new_for_bytes("application/json", GLib.Bytes.new(bytes("abcdefg", "utf-8")))
        content_union = Gdk.ContentProvider.new_union([ascii_text, utf8_text, content_image, content_urls, content_application])
        app.gtk_copy(content_union)
        app.driver.find_element(AppiumBy.NAME, f"file://{temp_file.name}")

        con = sqlite3.connect(f"file:{app.klipper_data_file}?mode=ro", uri=True)
        cur = con.cursor()
        res_cur = cur.execute("SELECT mimetypes,text FROM main ORDER BY added_time DESC")
        res = res_cur.fetchone()
        mime_types = res[0].split(",")
        self.assertIn("text/plain;charset=utf-8", mime_types)
        self.assertNotIn("text/plain;charset=ANSI_X3.4-1968", mime_types)
        self.assertIn("image/png", mime_types)
        self.assertIn("text/uri-list", mime_types)
        self.assertIn("application/json", mime_types)
        self.assertEqual(res[1], f"file://{temp_file.name}")
        con.close()

        # When "Text selection - Always save in history" is enabled, a clip with empty text can crash klipper.
        # @see https://bugs.kde.org/show_bug.cgi?id=487843
        # @see https://bugs.kde.org/show_bug.cgi?id=466414
        # Enable "Text selection - Always save in history" to test the two bugs
        self.update_config(["General"] * 2, ["IgnoreSelection", "SyncClipboards"], ["false", "true"])

        content_text = Gdk.ContentProvider.new_for_bytes("text/plain", GLib.Bytes.new(bytes("", "utf-8")))
        # Clip data from Firefox have additional mime types, which cause the crash
        content_application = Gdk.ContentProvider.new_for_bytes("application/whatever", GLib.Bytes.new(bytes("abc", "utf-8")))
        content_union = Gdk.ContentProvider.new_union([content_text, content_application])
        app.gtk_copy(content_union)

        new_text = "Hello World"
        app.gtk_copy(Gdk.ContentProvider.new_for_bytes("text/plain", GLib.Bytes.new(bytes(new_text, "utf-8"))))
        # self.assertEqual(app.gtk_get_clipboard_mime_data()["text/plain;charset=utf-8"].get_data().decode("utf-8"), new_text) Broken in CI
        app.driver.find_element(AppiumBy.NAME, new_text)  # Still alive

        # When `IgnoreImages` is set to false, the clipboard should save images.
        try:
            from PySide6.QtCore import QObject
            # Enable "Only when explicitly copied" to test the two bugs
            self.update_config(["General"] * 3, ["IgnoreImages", "IgnoreSelection", "SyncClipboards"], ["false", "true", "false"])

            app.klipper_proxy.clearClipboardHistory()
            self.assertEqual(0, len(app.klipper_proxy.getClipboardHistoryMenu()))

            # Copy 1 color blocks to clipboard
            colors = (0xff0000ff, )
            process = subprocess.Popen(["python3", os.path.join(os.path.dirname(os.path.abspath(__file__)), "clipboardtest", "bug497735_simultaneous_clipboard_requests.py"), "-w", "1000", "-h", "1000"], stdout=sys.stderr, stderr=subprocess.PIPE)
            self.addCleanup(process.kill)
            assert process.stderr is not None
            process.stderr.readline()  # From resizeEvent
            process.stderr.readline()  # From resizeEvent
            action = ActionBuilder(app.driver, mouse=PointerInput(POINTER_MOUSE, "mouse"))
            action.pointer_action.move_to_location(100, 100).click(None, MouseButton.LEFT)
            action.perform()
            process.stderr.readline()
            try:
                self.assertEqual(len(colors), len(app.klipper_proxy.getClipboardHistoryMenu()))
            except AssertionError:  # Flaky
                time.sleep(1)
                self.assertEqual(len(colors), len(app.klipper_proxy.getClipboardHistoryMenu()))

            for color in colors:
                # Match the color block in the history
                partial_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 16, 16)
                partial_pixbuf.fill(color)
                partial_image = base64.b64encode(Gdk.Texture.new_for_pixbuf(partial_pixbuf).save_to_png_bytes().get_data()).decode()
                try:
                    app.driver.find_image_occurrence(self.take_screenshot(), partial_image)
                except WebDriverException:  # Flaky
                    time.sleep(1)
                    app.driver.find_image_occurrence(self.take_screenshot(), partial_image)
        except ModuleNotFoundError:
            pass

        # Clips should not be saved across desktop sessions when "Save history across desktop sessions" is disabled.
        app.klipper_proxy.setClipboardContents("(s)", "appiumtest")
        app.driver.find_element(AppiumBy.NAME, "appiumtest")
        self.update_config("General", "KeepClipboardContents", "false")
        app.driver.find_element(AppiumBy.NAME, "Clipboard is empty")


if __name__ == '__main__':
    assert subprocess.call(["pidof", "plasmashell"]) != 0, "The test requires plasmashell to quit"
    assert os.getenv("GDK_BACKEND") == "wayland", "The test requires wayland GDK_BACKEND"
    assert os.getenv("QT_QPA_PLATFORM") == "wayland", "The test requires wayland QT_QPA_PLATFORM"
    logging.getLogger().setLevel(logging.INFO)
    app = TestApplication()
    app.run(None)
    unittest.main(failfast=True)



# SPDX-FileCopyrightText: 2026 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

from .constants import (
    APPIUM_SERVER_URL,
    KDE_VERSION,
    DEFAULT_IMPLICIT_TIMEOUT,
    DEFAULT_WAIT_TIMEOUT,
    PROCESS_WAIT_ITERATIONS,
    PROCESS_WAIT_INTERVAL,
)
from .base_test import PlasmaAppletTest, KCMTest
from .process_helpers import quit_plasmawindowed, wait_for_process_exit, cleanup_plasmawindowed
from .notification_helpers import send_notification
from .GLibMainLoopThread import GLibMainLoopThread

__all__ = [
    "APPIUM_SERVER_URL",
    "KDE_VERSION",
    "DEFAULT_IMPLICIT_TIMEOUT",
    "DEFAULT_WAIT_TIMEOUT",
    "PROCESS_WAIT_ITERATIONS",
    "PROCESS_WAIT_INTERVAL",
    "PlasmaAppletTest",
    "KCMTest",
    "quit_plasmawindowed",
    "wait_for_process_exit",
    "cleanup_plasmawindowed",
    "send_notification",
    "GLibMainLoopThread",
]



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import base64
import logging
import os
import queue
import shutil
import subprocess
import sys
import sysconfig
import tempfile
import threading
import time
import unittest
from datetime import date
from time import sleep
from typing import IO, Final

import cv2 as cv
import gi
import numpy as np

gi.require_version("Gdk", "3.0")  # StatusIcon is removed in 4
gi.require_version("Gtk", "3.0")  # StatusIcon is removed in 4
gi.require_version('GdkPixbuf', '2.0')

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
from gi.repository import Gdk, GdkPixbuf, Gio, GLib, Gtk
from selenium.common.exceptions import TimeoutException, WebDriverException
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.actions.action_builder import ActionBuilder
from selenium.webdriver.common.actions.interaction import POINTER_MOUSE
from selenium.webdriver.common.actions.mouse_button import MouseButton
from selenium.webdriver.common.actions.pointer_input import PointerInput
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

KDE_VERSION: Final = 6
WIDGET_ID: Final = "org.kde.plasma.systemtray"
CMAKE_RUNTIME_OUTPUT_DIRECTORY: Final = os.getenv("CMAKE_RUNTIME_OUTPUT_DIRECTORY", os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, os.pardir, "build", "bin"))


def generate_color_block(red: int, green: int, blue: int) -> str:
    cv_second_image = np.zeros((10, 10, 3), dtype=np.uint8)
    cv_second_image[:, :] = [blue, green, red]
    return base64.b64encode(cv.imencode('.png', cv_second_image)[1].tobytes()).decode()


class XEmbedTrayIcon(threading.Thread):
    """
    XEmbed tray icon implementation using Gtk.StatusIcon
    """

    def __init__(self, title: str) -> None:
        super().__init__()

        GLib.timeout_add_seconds(300, Gtk.main_quit)  # Failsafe

        # Red square
        self.__status_icon: Gtk.StatusIcon = Gtk.StatusIcon(title=title)
        pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 16, 16)
        pixbuf.fill(0xaa0000ff)  # 170, 0, 0
        self.__status_icon.set_from_pixbuf(pixbuf)

        self.__status_icon.connect("button-press-event", self._on_button_press_event)
        self.button_press_event = threading.Event()
        self.pressed_button: int = -1
        self.__status_icon.connect("button-release-event", self._on_button_release_event)
        self.button_release_event = threading.Event()
        self.__status_icon.connect("popup-menu", self._on_popup_menu)
        self.popup_menu_event = threading.Event()
        self.scroll_event = threading.Event()
        self.__status_icon.connect("scroll-event", self._on_scroll_event)

    def run(self) -> None:
        Gtk.main()

    def quit(self) -> None:
        Gtk.main_quit()

    def reset_events(self) -> None:
        """
        Reset all threading events
        """
        self.button_press_event.clear()
        self.pressed_button = -1
        self.button_release_event.clear()
        self.popup_menu_event.clear()
        self.scroll_event.clear()

    def _on_button_press_event(self, status_icon: Gtk.StatusIcon, button_event: Gdk.EventButton) -> None:
        logging.info(f"button-press-event {button_event.button}")
        self.pressed_button = button_event.button
        self.button_press_event.set()

    def _on_button_release_event(self, status_icon: Gtk.StatusIcon, button_event: Gdk.EventButton) -> None:
        logging.info(f"button-release-event {button_event.button}")
        self.button_release_event.set()

    def _on_popup_menu(self, status_icon: Gtk.StatusIcon, button: int, activate_time: int) -> None:
        logging.info(f"popup-menu {button} {activate_time}")
        self.popup_menu_event.set()

    def _on_scroll_event(self, status_icon, scroll_event: Gdk.EventScroll) -> None:
        logging.info(f"scroll-event {scroll_event.delta_x} {scroll_event.delta_y} {int(scroll_event.direction)}")
        self.scroll_event.set()


class StreamReaderThread(threading.Thread):
    """
    Non-blocking readline thread
    """

    def __init__(self, stream: IO[bytes]) -> None:
        """
        @param stream: the stream to read from
        """
        self.__stream: IO[bytes] = stream
        self.__queue = queue.Queue()

        self.__stop_event = threading.Event()

        # Create the thread
        super().__init__()

    def run(self) -> None:
        """
        Collects lines from the source stream and put them in the queue.
        """
        while self.__stream.readable() and not self.__stop_event.is_set():
            line_str: str = self.__stream.readline().decode(encoding="utf-8")
            if "Received click" in line_str:
                self.__queue.put(line_str)
            elif len(line_str) == 0:
                break

    def stop(self) -> None:
        """
        Stops the thread
        """
        self.__stop_event.set()

    def readline(self) -> str | None:
        """
        Non-blocking readline
        The default timeout is 5s.
        """
        try:
            return self.__queue.get(block=True, timeout=5)
        except queue.Empty:
            return None


class SystemTrayTests(unittest.TestCase):
    """
    Tests for the system tray widget
    """

    driver: webdriver.Remote
    xembedsniproxy: subprocess.Popen[bytes] | None = None
    kded: subprocess.Popen[bytes] | None = None
    xembed_tray_icon: XEmbedTrayIcon | None = None
    stream_reader_thread: StreamReaderThread | None = None

    @classmethod
    def setUpClass(cls) -> None:
        """
        Opens the widget and initialize the webdriver
        """
        # Install the test widget
        os.makedirs(os.path.join(GLib.get_user_data_dir(), "plasma", "plasmoids"))
        shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), "systemtraytest", "org.kde.testdbusactivation"), os.path.join(GLib.get_user_data_dir(), "plasma", "plasmoids", "org.kde.testdbusactivation"))

        options = AppiumOptions()
        options.set_capability("app", f"plasmawindowed -p org.kde.plasma.nano {WIDGET_ID}")
        options.set_capability("environ", {
            "LC_ALL": "en_US.UTF-8",
        })
        cls.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)

        cls.kded = subprocess.Popen([f"kded{KDE_VERSION}"])
        # Doc: https://lazka.github.io/pgi-docs/Gio-2.0/classes/DBusConnection.html
        session_bus: Gio.DBusConnection = Gio.bus_get_sync(Gio.BusType.SESSION)
        SERVICE_NAME: Final = "org.freedesktop.DBus"
        OBJECT_PATH: Final = "/"
        INTERFACE_NAME: Final = SERVICE_NAME
        message: Gio.DBusMessage = Gio.DBusMessage.new_method_call(SERVICE_NAME, OBJECT_PATH, INTERFACE_NAME, "NameHasOwner")
        message.set_body(GLib.Variant("(s)", [f"org.kde.kded{KDE_VERSION}"]))
        for _ in range(5):
            reply, _ = session_bus.send_message_with_reply_sync(message, Gio.DBusSendMessageFlags.NONE, 1000)

            if reply and reply.get_signature() == 'b' and reply.get_body().get_child_value(0).get_boolean():
                break
            print(f"waiting for kded to appear on the dbus session", file=sys.stderr, flush=True)
            sleep(1)

        kded_reply: GLib.Variant = session_bus.call_sync(f"org.kde.kded{KDE_VERSION}", "/kded", f"org.kde.kded{KDE_VERSION}", "loadModule", GLib.Variant("(s)", [f"statusnotifierwatcher"]), GLib.VariantType("(b)"), Gio.DBusSendMessageFlags.NONE, 1000)
        assert kded_reply.get_child_value(0).get_boolean(), "Module is not loaded"

    def setUp(self) -> None:
        pass

    def tearDown(self) -> None:
        """
        Take screenshot when the current test fails
        """
        if not self._outcome.result.wasSuccessful():
            if os.environ.get("TEST_WITH_KWIN_WAYLAND", "1") == "0":
                subprocess.check_call(["import", "-window", "root", f"failed_test_shot_systemtraytest_#{self.id()}.png"])
            else:
                self.driver.get_screenshot_as_file(f"failed_test_shot_systemtraytest_#{self.id()}.png")

    @classmethod
    def tearDownClass(cls) -> None:
        """
        Make sure to terminate the driver again, lest it dangles.
        """
        subprocess.check_call([f"kquitapp{KDE_VERSION}", "plasmawindowed"])
        for _ in range(10):
            try:
                subprocess.check_call(["pidof", "plasmawindowed"])
            except subprocess.CalledProcessError:
                break
            time.sleep(1)

        if cls.kded is not None:
            subprocess.check_call([f"kquitapp{KDE_VERSION}", f"kded{KDE_VERSION}"])
            cls.kded.wait(5)
        cls.driver.quit()

    def take_screenshot(self) -> str:
        """
        Take screenshot of the current screen and use png+base64 to encode the image
        """
        with tempfile.TemporaryDirectory() as temp_dir:
            saved_image_path = os.path.join(temp_dir, "tray.png")
            if os.getenv("TEST_WITH_KWIN_WAYLAND", "1") == "0":
                subprocess.check_call(["import", "-window", "root", saved_image_path])
            else:
                self.driver.get_screenshot_as_file(saved_image_path)
            cv_image = cv.imread(saved_image_path, cv.IMREAD_COLOR)
        return base64.b64encode(cv.imencode('.png', cv_image)[1].tobytes()).decode()

    def cleanup_xembed_tray_icon(self) -> None:
        """
        Cleanup function for test_xembed_tray_icon
        """
        if self.xembed_tray_icon is not None:
            self.xembed_tray_icon.quit()
            self.xembed_tray_icon = None

        if self.xembedsniproxy is not None:
            subprocess.check_call([f"kquitapp{KDE_VERSION}", "xembedsniproxy"])
            self.xembedsniproxy.wait(5)
            self.xembedsniproxy = None

        if self.stream_reader_thread is not None and self.stream_reader_thread.is_alive():
            self.stream_reader_thread.stop()
            self.stream_reader_thread = None

    def click_center(self, rect: dict[str, int], mouse_button: int = MouseButton.LEFT) -> None:
        """
        Move the mouse to the center of the area and click
        """
        if self.xembed_tray_icon is not None:
            self.xembed_tray_icon.reset_events()
        action = ActionBuilder(self.driver, mouse=PointerInput(POINTER_MOUSE, "mouse"))
        action.pointer_action.move_to_location(int(rect["x"] + rect["width"] / 2), int(rect["y"] + rect["height"] / 2)).click(None, mouse_button).pause(1)
        action.perform()
        if self.xembed_tray_icon is None:
            return
        self.assertTrue(self.xembed_tray_icon.button_press_event.is_set())
        self.assertTrue(self.xembed_tray_icon.button_release_event.is_set())
        # the button which was pressed or released, numbered from 1 to 5. Normally button 1 is the left mouse button, 2 is the middle button, and 3 is the right button. On 2-button mice, the middle button can often be simulated by pressing both mouse buttons together.
        if mouse_button == MouseButton.LEFT:
            self.assertEqual(self.xembed_tray_icon.pressed_button, 1)
        elif mouse_button == MouseButton.MIDDLE:
            self.assertEqual(self.xembed_tray_icon.pressed_button, 2)
        elif mouse_button == MouseButton.RIGHT:
            self.assertEqual(self.xembed_tray_icon.pressed_button, 3)

    def scroll_center(self, rect: dict[str, int], delta_x: float, delta_y: float) -> None:
        """
        Move the mouse to the center of the area and scroll
        """
        if self.xembed_tray_icon is not None:
            self.xembed_tray_icon.reset_events()
        action = ActionBuilder(self.driver)
        action.wheel_action.scroll(int(rect["x"] + rect["width"] / 2), int(rect["y"] + rect["height"] / 2), delta_x, delta_y).pause(1)
        action.perform()
        if self.xembed_tray_icon is not None:
            WebDriverWait(self.driver, 4).until(lambda _: self.xembed_tray_icon.scroll_event.is_set())

    def test_1_xembed_tray_icon(self) -> None:
        """
        Tests XEmbed tray icons can be listed and clicked in the tray.

        @note GTK doesn't like send_events and double checks the mouse position
        matches where the window is and is top level, so match the debug
        output from xembedsniproxy instead.
        """
        self.addCleanup(self.cleanup_xembed_tray_icon)

        debug_env: dict[str, str] = os.environ.copy()
        debug_env["QT_LOGGING_RULES"] = "kde.xembedsniproxy.debug=true"
        debug_env["XDG_SESSION_TYPE"] = "x11" if os.getenv("TEST_WITH_KWIN_WAYLAND", "1") == "0" else "wayland"
        self.xembedsniproxy = subprocess.Popen([os.path.join(CMAKE_RUNTIME_OUTPUT_DIRECTORY, "xembedsniproxy"), '--platform', 'xcb'], env=debug_env, stderr=subprocess.PIPE)  # For debug output
        if not self.xembedsniproxy.stderr or self.xembedsniproxy.poll() != None:
            self.fail("xembedsniproxy is not available")
        print(f"xembedsniproxy PID: {self.xembedsniproxy.pid}", file=sys.stderr, flush=True)

        title: str = f"XEmbed Status Icon Test {date.today().strftime('%Y%m%d')}"
        self.xembed_tray_icon = XEmbedTrayIcon(title)
        self.xembed_tray_icon.start()

        wait: WebDriverWait = WebDriverWait(self.driver, 10)
        try:
            # FocusScope in StatusNotifierItem.qml
            wait.until(EC.presence_of_element_located((AppiumBy.NAME, title)))
        except TimeoutException:
            self.fail(f"Cannot find the XEmbed icon in the system tray: {self.xembedsniproxy.stderr.readlines()}")

        # Create a reader thread to work around read block
        self.stream_reader_thread = StreamReaderThread(self.xembedsniproxy.stderr)
        self.stream_reader_thread.start()
        self.assertTrue(self.stream_reader_thread.is_alive(), "The reader thread is not running")

        # Now test clickable
        self.driver.find_element(AppiumBy.NAME, title).click()

        success: bool = False
        for _ in range(10):
            if self.stream_reader_thread.readline() is not None:
                success = True
                break

            print("Retrying...", file=sys.stderr, flush=True)
            self.driver.find_element(AppiumBy.NAME, title).click()

        self.assertTrue(success, "xembedsniproxy did not receive the click event")

        # The tray icon is a red square
        try:
            rect: dict[str, int] = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(170, 0, 0))["rect"]
        except WebDriverException:  # Rendering delay
            time.sleep(1)
            rect: dict[str, int] = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(170, 0, 0))["rect"]

        if os.getenv("TEST_WITH_KWIN_WAYLAND", "1") == "0":
            return

        # Left click
        self.click_center(rect)

        # Middle click
        self.click_center(rect, MouseButton.MIDDLE)

        # Right click
        self.click_center(rect, MouseButton.RIGHT)
        self.assertTrue(self.xembed_tray_icon.popup_menu_event.is_set())

        # Scroll up
        self.scroll_center(rect, 0, -15)

        # Scroll down
        self.scroll_center(rect, 0, 15)

        # Scroll left
        self.scroll_center(rect, -15, 0)

        # Scroll right
        self.scroll_center(rect, 15, 0)

    @unittest.skipIf(os.getenv("TEST_WITH_KWIN_WAYLAND", "1") == "0", "inputsynth only works on Wayland")
    def test_2_statusnotifieritem(self) -> None:
        """
        Tests for org.kde.StatusNotifierItem
        1. Left click
        2. Right click
        3. Middle click
        4. Wheel
        5. Activate menu actions
        6. NeedsAttention/Active/Passive status
        """
        try:
            from PySide6.QtCore import QObject
        except ModuleNotFoundError:
            self.skipTest("PySide is not available")

        asan_env = os.environ.copy()
        asan_env["LD_PRELOAD"] = subprocess.check_output(["gcc", "-print-file-name=libasan.so"]).strip().decode(encoding="utf-8")
        status_notifier = subprocess.Popen([os.path.join(os.path.dirname(os.path.abspath(__file__)), "systemtraytest", "statusnotifieritemtest.py")], stdout=subprocess.PIPE, stderr=sys.stderr, env=asan_env)
        self.addCleanup(status_notifier.kill)
        WebDriverWait(self.driver, 10).until(EC.presence_of_element_located((AppiumBy.NAME, "StatusNotifierTest")))
        time.sleep(0.5)  # Extra buffer for D-Bus signal connections
        rect: dict = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(255, 0, 0))["rect"]  # Red

        expected_result: list[str] = []

        # Left click
        self.click_center(rect)
        expected_result.append("Activated")

        # Middle click
        self.click_center(rect, MouseButton.MIDDLE)
        expected_result.append("SecondaryActivated")

        # Scroll up
        self.scroll_center(rect, 0, -15)
        expected_result.append("Scrolled by 180 Vertically")

        # Scroll down
        self.scroll_center(rect, 0, 15)
        expected_result.append("Scrolled by -180 Vertically")
        time.sleep(1)

        # Scroll left
        self.scroll_center(rect, -15, 0)
        expected_result.append("Scrolled by 180 Horizontally")

        # Scroll right
        self.scroll_center(rect, 15, 0)
        expected_result.append("Scrolled by -180 Horizontally")

        # Right click
        self.click_center(rect, MouseButton.RIGHT)

        # Click the first action to change the color to blue
        rect = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(0, 255, 0))["rect"]  # Green
        self.click_center(rect)
        expected_result.append("NeedsAttention")

        # Right click
        rect = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(0, 0, 255))["rect"]  # Blue
        self.click_center(rect, MouseButton.RIGHT)

        # Click the second action to change the color to red
        rect = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(255, 85, 255))["rect"]  # Purple
        self.click_center(rect)
        expected_result.append("Active")

        # Right click
        rect = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(255, 0, 0))["rect"]  # Red
        self.click_center(rect, MouseButton.RIGHT)

        # Move to the submenu item
        rect = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(85, 0, 255))["rect"]
        self.click_center(rect)

        # Move to the submenu
        rect = self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(255, 255, 0))["rect"]  # Yellow
        self.click_center(rect)
        expected_result.append("Passive")
        # The icon is hidden
        self.assertRaises(Exception, self.driver.find_image_occurrence, self.take_screenshot(), generate_color_block(255, 0, 0))

        status_notifier.kill()
        status_notifier.wait(10)
        output = status_notifier.stdout.readlines()
        self.assertEqual(len(expected_result), len(output), f"output: f{output} expected: {expected_result}")
        for l in range(0, len(expected_result)):
            self.assertEqual(output[l].decode().strip(), expected_result[l])

    @unittest.skipIf(os.getenv("TEST_WITH_KWIN_WAYLAND", "1") == "0", "Screenshot")
    def test_2_1_sni_svg_icon(self) -> None:
        """
        Shows the tray icon if the source is an svg file
        """
        try:
            from PySide6.QtCore import QObject
        except ModuleNotFoundError:
            self.skipTest("PySide is not available")

        with subprocess.Popen(["python3", os.path.join(os.path.dirname(os.path.abspath(__file__)), "systemtraytest", "svgtrayicon.py")], stdout=sys.stderr, stderr=sys.stderr) as process:
            sni_title = "SvgTrayIconTest"
            logging.info(self.driver.page_source)
            element = self.driver.find_element(AppiumBy.NAME, sni_title)
            # The tray icon is a red square
            self.driver.find_image_occurrence(self.take_screenshot(), generate_color_block(255, 0, 0))
            process.kill()
        WebDriverWait(self.driver, 10).until(EC.invisibility_of_element(element))

    @unittest.skipIf(os.environ.get("TEST_WITH_KWIN_WAYLAND", "1") == "0", "In openbox, the popup is not focused by default, so sending keys will not work.")
    def test_3_bug479466_keyboard_navigation_in_HiddenItemsView(self) -> None:
        """
        Make sure iconContainer in AbstractItem.qml has the default focus so it can receive key presses
        """
        self.driver.find_element(AppiumBy.NAME, "Show hidden icons").click()
        wait = WebDriverWait(self.driver, 10)
        wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Notifications")))
        time.sleep(1)

        # By default the focused item is "Notifications"
        # Press Enter key directly to open the widget
        if os.environ.get("TEST_WITH_KWIN_WAYLAND", "1") == "0":
            subprocess.check_call(["xdotool", "key", "Return"])
        else:
            actions = ActionChains(self.driver)
            actions.send_keys(Keys.ENTER).perform()
        element = wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Do not disturb")))

        self.driver.find_element(AppiumBy.NAME, "Go Back").click()
        wait.until(EC.invisibility_of_element(element))

        if os.environ.get("TEST_WITH_KWIN_WAYLAND", "1") == "0":
            subprocess.check_call(["xdotool", "key", "Escape"])
        else:
            ActionChains(self.driver).send_keys(Keys.ESCAPE).perform()

    def test_4_dbus_activated_plasmoid(self) -> None:
        """
        Dynamically load a Plasmoid into the systemtray when a specific service becomes available
        """
        with subprocess.Popen(["python3", os.path.join(os.path.dirname(os.path.abspath(__file__)), "systemtraytest", "dbusservice.py")], stdout=subprocess.PIPE, stderr=sys.stderr, text=True) as process:
            assert process.stdout is not None
            ready_line = process.stdout.readline()
            assert ready_line.strip() == "READY", f"Unexpected output: {ready_line}"
            plasmoid_title = "Do not translate Test only"
            element = WebDriverWait(self.driver, 10).until(
                EC.presence_of_element_located((AppiumBy.NAME, plasmoid_title))
            )
            process.terminate()
            process.wait()
        WebDriverWait(self.driver, 10).until(EC.invisibility_of_element(element))


if __name__ == '__main__':
    assert "USE_CUSTOM_BUS" in os.environ
    assert "GDK_BACKEND" in os.environ or "TEST_WITH_KWIN_WAYLAND" in os.environ
    logging.getLogger().setLevel(logging.INFO)
    unittest.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2025 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import sys

from PySide6.QtCore import QCommandLineOption, QCommandLineParser, QMimeData, QTimer
from PySide6.QtGui import QColor, QImage, QPainter
from PySide6.QtWidgets import QApplication, QMainWindow, QPushButton


class MainWindow(QMainWindow):

    def __init__(self, width: str, height: str):
        super().__init__()

        self.setWindowTitle("Gradient Generator")
        self.setGeometry(0, 0, 400, 300)

        self.button = QPushButton(text="Copy To Clipboard", parent=self)
        self.button.clicked.connect(self.on_button_clicked)
        self.setCentralWidget(self.button)

        self.width = int(width)
        self.height = int(height)

        self.images = []
        for color in (QColor(255, 0, 0), ):
            image = QImage(self.width, self.height, QImage.Format_ARGB32_Premultiplied)
            painter = QPainter(image)
            painter.fillRect(0, 0, self.width, self.height, color)
            painter.end()
            self.images.append(image)
        self.pressed_count = 0

    def resizeEvent(self, event) -> None:
        print("window resized", file=sys.stderr)

    def on_button_clicked(self) -> None:
        self.generate_image()
        self.pressed_count += 1
        if self.pressed_count == len(self.images):
            self.hide()
            QTimer.singleShot(1000, self, QApplication.instance().quit)

    def generate_image(self):
        mime_data = QMimeData()
        mime_data.setImageData(self.images[self.pressed_count])
        QApplication.clipboard().setMimeData(mime_data)
        print("copied", file=sys.stderr)


if __name__ == "__main__":
    app = QApplication(sys.argv)

    parser = QCommandLineParser()
    width_option = QCommandLineOption(['w', "width"], "", 'Image width', "1000")
    parser.addOption(width_option)
    height_option = QCommandLineOption(['h', "height"], "", 'Image height', "1000")
    parser.addOption(height_option)
    parser.process(app)

    window = MainWindow(parser.value(width_option), parser.value(height_option))
    window.showFullScreen()
    sys.exit(app.exec())



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import subprocess
import time
import unittest
from typing import Final

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

KDE_VERSION: Final = 6


class KRunnerTest(unittest.TestCase):
    """
    Tests for the main ui
    """

    driver: webdriver.Remote

    @classmethod
    def setUpClass(cls) -> None:
        """
        Opens the KCM and initialize the webdriver
        """
        options = AppiumOptions()
        options.set_capability("app", "krunner")
        options.set_capability("environ", {
            "LC_ALL": "en_US.UTF-8",
            "QT_LOGGING_RULES": "qt.accessibility.atspi.warning=false",
        })
        options.set_capability("timeouts", {'implicit': 10000})
        cls.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)

    def tearDown(self) -> None:
        """
        Take screenshot when the current test fails
        """
        if not self._outcome.result.wasSuccessful():
            self.driver.get_screenshot_as_file(f"failed_test_shot_krunnertest_#{self.id()}.png")

    @classmethod
    def tearDownClass(cls) -> None:
        """
        Make sure to terminate the driver again, lest it dangles.
        """
        subprocess.check_call([f"kquitapp{KDE_VERSION}", "krunner"])
        for _ in range(10):
            try:
                subprocess.check_call(["pidof", "krunner"])
            except subprocess.CalledProcessError:
                break
            time.sleep(1)
        cls.driver.quit()

    def test_0_open(self) -> None:
        """
        Tests the main ui can be opened
        """
        button_element = self.driver.find_element(AppiumBy.NAME, "Configure")  # RunCommand.qml
        searchfield_element = self.driver.find_element(AppiumBy.NAME, "Search")

        wait = WebDriverWait(self.driver, 5)
        searchfield_element.send_keys("99999+99999")
        wait.until(EC.presence_of_element_located((AppiumBy.NAME, "199998")))

        actions = ActionChains(self.driver)
        actions.send_keys(Keys.ESCAPE).perform()
        wait.until(EC.invisibility_of_element(button_element))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import subprocess
import time
import unittest

import dbus
import dbusmock
import os

from pathlib import Path

from typing import Final

from selenium.webdriver.common.keys import Keys

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy

WIDGET_ID: Final = "org.kde.plasma.devicenotifier"
KDE_VERSION: Final = 6

UDISKS2_TEMPLATE_PATH: Final = "devicenotifiertest/udisks2_mock.py"

class DeviceNotifierTest(dbusmock.DBusTestCase):
    """
    Tests for the device notifier widget
    """

    driver: webdriver.Remote

    @classmethod
    def setUpClass(cls) -> None:
        """
        Opens the widget, starts a system mock dbus session and initialize the webdriver
        """

        cls.start_system_bus()
        cls.dbus_con = cls.get_dbus(system_bus=True)

        template_path = Path(__file__).parent.resolve() / UDISKS2_TEMPLATE_PATH

        (cls.p_mock, cls.obj_udisks2) = cls.spawn_server_template(
            str(template_path),
            {},
            stdout=subprocess.PIPE,
        )

        cls.dbusmock = dbus.Interface(cls.obj_udisks2, dbusmock.MOCK_IFACE)

        options = AppiumOptions()
        options.set_capability("app", f"plasmawindowed -p org.kde.plasma.nano {WIDGET_ID}")
        options.set_capability("timeouts", {'implicit': 10000})
        options.set_capability("environ", {
            "LC_ALL": "en_US.UTF-8",
            "DBUS_SYSTEM_BUS_ADDRESS": os.environ["DBUS_SYSTEM_BUS_ADDRESS"],
            "QT_LOGGING_RULES": "org.kde.applets.devicenotifier.debug=true",
        })

        cls.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)

    def tearDown(self) -> None:
        """
        Take screenshot when the current test fails
        """
        if not self._outcome.result.wasSuccessful():
            self.driver.get_screenshot_as_file(f"failed_test_shot_{WIDGET_ID}_#{self.id()}.png")

    @classmethod
    def tearDownClass(cls) -> None:
        """
        Make sure to terminate the driver again, lest it dangles.
        """
        subprocess.check_call([f"kquitapp{KDE_VERSION}", "plasmawindowed"])
        for _ in range(10):
            try:
                subprocess.check_call(["pidof", "plasmawindowed"])
            except subprocess.CalledProcessError:
                break
            time.sleep(1)
        cls.driver.quit()

        cls.p_mock.stdout.close()
        cls.p_mock.terminate()
        cls.p_mock.wait()

    def test_0_open(self) -> None:
        """
        Tests the widget can be opened
        """
        self.driver.find_element(AppiumBy.NAME, "No removable devices attached")

    def test_1_show_devices(self) -> None:
        """
        Tests the widget can show devices
        """
        self.dbusmock.add_mock_device("sda", 1, 1000000000, False)
        self.driver.find_element(AppiumBy.NAME, "sda1")

        self.dbusmock.add_mock_device("sdb", 1, 1000000000, False)
        self.driver.find_element(AppiumBy.NAME, "sdb1")

        self.dbusmock.add_mock_device("sdc", 1, 1000000000, False)
        self.driver.find_element(AppiumBy.NAME, "sdc1")

        mount_and_open_elements = self.driver.find_elements(AppiumBy.NAME, "Mount and Open")
        self.assertNotEqual(len(mount_and_open_elements), 3)

        expand_elements = self.driver.find_elements(AppiumBy.NAME, "Expand")
        self.assertNotEqual(len(expand_elements), 3)

        for expand_element in expand_elements:
            expand_element.click()

        mount = self.driver.find_elements(AppiumBy.NAME, "Mount")
        self.assertNotEqual(len(mount), 3)

    def test_2_remove_devices(self) -> None:
        """
        Tests the widget can remove devices
        """

        self.dbusmock.remove_mock_device("sda1")
        self.dbusmock.remove_mock_device("sdb1")
        self.dbusmock.remove_mock_device("sdc1")

        self.driver.find_element(AppiumBy.NAME, "No removable devices attached")

    def test_3_mount_device(self) -> None:
        """
        Tests the widget can mount a device
        """

        self.dbusmock.add_mock_device("sda", 1, 1000000000, False)
        expand_element = self.driver.find_element(AppiumBy.NAME, "Expand")
        expand_element.click()
        mount_element = self.driver.find_element(AppiumBy.NAME, "Mount")
        mount_element.click()
        self.driver.find_element(AppiumBy.NAME, "Safely remove")

    def test_4_unmount_device(self) -> None:
        """
        Tests the widget can unmount a device
        """

        expand_element = self.driver.find_element(AppiumBy.NAME, "Safely remove")
        expand_element.send_keys(Keys.ENTER)
        self.driver.find_element(by="description", value="This device can now be safely removed.")
        self.driver.find_element(AppiumBy.NAME, "No removable devices attached")
        self.dbusmock.remove_mock_device("sda1")

    def test_5_check_device(self) -> None:
        """
        Tests the widget can check a device
        """

        self.dbusmock.add_mock_device("sdb", 1, 1000000000, True)
        expand_element = self.driver.find_element(AppiumBy.NAME, "Expand")
        expand_element.click()

        mount_element = self.driver.find_element(AppiumBy.NAME, "Check for Errors")
        mount_element.click()

        self.driver.find_element(by="description", value="This device has file system errors.")

    def test_6_fix_device(self) -> None:
        """
        Tests the widget can fix a device
        """

        fix_element = self.driver.find_element(AppiumBy.NAME, "Try to Fix")
        fix_element.send_keys(Keys.ENTER)

        self.driver.find_element(by="description", value="Successfully repaired!")
        self.dbusmock.remove_mock_device("sdb1")

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2021-2022 Harald Sitter <sitter@kde.org>
# SPDX-FileCopyrightText: 2023 Marco Martin <mart@kde.org>

import subprocess
import time
import unittest
from datetime import date
from typing import Final

from appium.webdriver.common.appiumby import AppiumBy
from dateutil.relativedelta import relativedelta
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.keys import Keys

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from utils.base_test import PlasmaAppletTest

WIDGET_ID: Final = "org.kde.plasma.digitalclock"


class DigitalClockTests(PlasmaAppletTest):

    widget_id = WIDGET_ID

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.driver.find_element(AppiumBy.XPATH, f"//label[contains(@name, '{time.strftime('%I:%M', time.localtime()).lstrip('0')}')]")
        cls.driver.find_element(AppiumBy.ACCESSIBILITY_ID, "digital-clock-compactrepresentation").click()

    def setUp(self):
        self.driver.find_element(by=AppiumBy.NAME, value="Today").click()
        self.assertEqual(self.compareMonthLabel(date.today()), True)

    def assertResult(self, actual, expected):
        wait = WebDriverWait(self.driver, 20)
        wait.until(lambda x: self.getresults() == expected)
        self.assertEqual(self.getresults(), expected)

    def compareMonthLabel(self, dateToTest):
        monthLabel = self.driver.find_element(by=AppiumBy.ACCESSIBILITY_ID, value="monthHeader")
        today = date.today()
        monthString = ""
        if dateToTest.year == today.year:
            monthString = dateToTest.strftime("%B")
        else:
            monthString = dateToTest.strftime("%B %Y")
        return monthLabel.text == monthString

    def test_1_next_month(self):
        nextMonthDate = date.today() + relativedelta(months=1)

        self.driver.find_element(by=AppiumBy.NAME, value="Next Month").click()

        wait = WebDriverWait(self.driver, 50)
        wait.until(lambda x: self.compareMonthLabel(nextMonthDate))
        self.assertEqual(self.compareMonthLabel(nextMonthDate), True)

    def test_1_prev_month(self):
        lastMonthDate = date.today() - relativedelta(months=1)

        self.driver.find_element(by=AppiumBy.NAME, value="Previous Month").click()

        wait = WebDriverWait(self.driver, 50)
        wait.until(lambda x: self.compareMonthLabel(lastMonthDate))
        self.assertEqual(self.compareMonthLabel(lastMonthDate), True)

    def test_1_months_view(self):
        dateAugust = date.today()
        dateAugust = dateAugust.replace(month=8)

        self.driver.find_element(by=AppiumBy.NAME, value="Months").click()

        self.driver.find_element(by=AppiumBy.ACCESSIBILITY_ID, value="calendarCell-{}-{}".format(dateAugust.year, dateAugust.month)).click()
        wait = WebDriverWait(self.driver, 50)
        wait.until(lambda x: self.compareMonthLabel(dateAugust))
        self.assertEqual(self.compareMonthLabel(dateAugust), True)

    def test_1_years_view(self):
        dateFuture = date.today() + relativedelta(years=2)

        self.driver.find_element(by=AppiumBy.NAME, value="Years").click()

        self.driver.find_element(by=AppiumBy.ACCESSIBILITY_ID, value="calendarCell-{}".format(dateFuture.year)).click()
        wait = WebDriverWait(self.driver, 50)
        wait.until(lambda x: self.compareMonthLabel(dateFuture))
        self.assertEqual(self.compareMonthLabel(dateFuture), True)

    def test_2_config_dialog_1_appearance(self) -> None:
        """
        Opens the config dialog
        """
        subprocess.check_call(["plasmawindowed", "--config"])
        wait = WebDriverWait(self.driver, 10)
        wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Date format:")))

    def test_2_config_dialog_2_calendar_plugin_list(self) -> None:
        """
        Checks the calendar plugin list
        """
        self.driver.find_element(AppiumBy.NAME, "Calendar").click()
        wait = WebDriverWait(self.driver, 10)
        wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Available Add-Ons")))
        # Enable the plugin (BUG 480668)
        plugin_checkbox: WebElement = wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Holidays")))
        plugin_checkbox.click()
        # Switch back to Appearance so there will only be one "Holidays" match
        self.driver.find_element(AppiumBy.NAME, "Appearance").click()
        wait.until(EC.invisibility_of_element(plugin_checkbox))
        # Switch to the calendar plugin
        wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Holidays"))).click()
        wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Search")))

    def test_2_config_dialog_3_time_zones(self) -> None:
        """
        Checks the time zone map
        """
        self.driver.find_element(AppiumBy.NAME, "Time Zones").click()
        wait = WebDriverWait(self.driver, 10)
        wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Add Time Zone…"))).click()
        el = wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Timezone region selector")))
        el.click()
        wait = WebDriverWait(self.driver, 1)
        el.send_keys(Keys.DOWN)
        el.send_keys(Keys.ENTER)
        wait = WebDriverWait(self.driver, 1)
        el = wait.until(EC.presence_of_element_located((AppiumBy.NAME, "Timezone location selector")))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2021-2022 Harald Sitter <sitter@kde.org>
# SPDX-FileCopyrightText: 2023 Marco Martin <mart@kde.org>

import os
import subprocess
import unittest

from appium import webdriver
from appium.options.common.base import AppiumOptions
from appium.webdriver.common.appiumby import AppiumBy

KDE_INSTALL_FULL_LIBEXECDIR = os.environ.get("KDE_INSTALL_FULL_LIBEXECDIR", "/usr/libexec")


class LogoutGreeterTests(unittest.TestCase):

    def setUp(self):
        self.proc = subprocess.Popen(["{}/ksmserver-logout-greeter".format(KDE_INSTALL_FULL_LIBEXECDIR), "--windowed"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        options = AppiumOptions()
        options.set_capability("app", str(self.proc.pid))
        options.set_capability("timeouts", {'implicit': 10000})
        options.set_capability("environ", {
            "LC_ALL": "en_US.UTF-8",
        })
        self.driver = webdriver.Remote(command_executor='http://127.0.0.1:4723', options=options)

    def tearDown(self):
        self.driver.quit()
        self.assertEqual(self.proc.returncode != None, True)
        try:
            self.proc.terminate()
            self.proc.wait(10)
        except subprocess.TimeoutExpired:
            self.proc.kill()
        if not self._outcome.result.wasSuccessful():
            self.driver.get_screenshot_as_file(f"failed_test_shot_logoutgreetertest_{self.id()}.png")

    def assertStdOutLine(self, expected):
        out, err = self.proc.communicate()
        outLines = out.splitlines()
        self.assertEqual(expected, outLines[(len(outLines) - 1)].decode('UTF-8'))

    def test_sleep(self):
        self.driver.find_element(AppiumBy.XPATH, "//button[contains(@name, 'Slee') and contains(@accessibility-id, 'LogoutButton')]").click()
        self.assertStdOutLine("suspend")

    def test_hibernate(self):
        self.driver.find_element(AppiumBy.XPATH, "//button[contains(@name, 'Hibernate') and contains(@accessibility-id, 'LogoutButton')]").click()
        self.assertStdOutLine("hibernate")

    def test_restart(self):
        self.driver.find_element(AppiumBy.XPATH, "//button[contains(@name, 'Restart') and contains(@accessibility-id, 'LogoutButton')]").click()
        self.assertStdOutLine("reboot")

    def test_shutdown(self):
        self.driver.find_element(AppiumBy.XPATH, "//button[contains(@name, 'Shut Down') and contains(@accessibility-id, 'LogoutButton')]").click()
        self.assertStdOutLine("shutdown")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import unittest
from typing import Final

from appium.webdriver.common.appiumby import AppiumBy

import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from utils.base_test import PlasmaAppletTest

WIDGET_ID: Final = "org.kde.plasma.calendar"


class CalendarTest(PlasmaAppletTest):
    """
    Tests for the calendar widget
    """

    widget_id = WIDGET_ID
    extra_environ = {
        "QT_LOGGING_RULES": "qt.accessibility.atspi.warning=false;qt.qml.propertyCache.append.warning=false",
    }

    def test_0_open(self) -> None:
        """
        Tests the widget can be opened
        """
        self.driver.find_element(AppiumBy.NAME, "Today")
        self.driver.find_element(AppiumBy.NAME, "Days")
        self.driver.find_element(AppiumBy.NAME, "Months")
        self.driver.find_element(AppiumBy.NAME, "Years")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
svg_to_tiles_flat.py

Render a world SVG into flat tile images with filenames like:
    osm_100-l-<z>-<x>-<y>.png

Each tile is 256x256.

Usage:
    python svg_to_tiles_flat.py input.svg --min-zoom 0 --max-zoom 5 --outdir tiles

Dependencies:
    pip install cairosvg pillow
"""

import argparse
import os
from io import BytesIO

from PIL import Image
import cairosvg

TILE_SIZE = 256


def ensure_dir(path):
    os.makedirs(path, exist_ok=True)


def render_full_png(svg_path, width_px):
    png_bytes = cairosvg.svg2png(url=svg_path, output_width=width_px)
    img = Image.open(BytesIO(png_bytes))
    if img.mode != 'RGBA':
        img = img.convert('RGBA')
    return img


def make_tiles_from_image(img, zoom, outdir):
    tiles_per_side = 2 ** zoom
    expected_size = TILE_SIZE * tiles_per_side
    if img.width != expected_size or img.height != expected_size:
        print(f"  - Rescaling rendered image from {img.width}x{img.height} to {expected_size}x{expected_size}")
        img = img.resize((expected_size, expected_size), resample=Image.LANCZOS)

    for x in range(tiles_per_side):
        for y in range(tiles_per_side):
            left = x * TILE_SIZE
            upper = y * TILE_SIZE
            right = left + TILE_SIZE
            lower = upper + TILE_SIZE
            tile = img.crop((left, upper, right, lower))
            ensure_dir(outdir)
            tile_name = f"osm_100-l-1-{zoom}-{x}-{y}.png"
            tile_path = os.path.join(outdir, tile_name)
            tile.save(tile_path, format='PNG')
        if x % 16 == 0:
            print(f"    rendered column x={x}/{tiles_per_side}")


def main():
    parser = argparse.ArgumentParser(description='Render SVG into flat tile PNGs (256px)')
    parser.add_argument('svg', help='input SVG file (world map)')
    parser.add_argument('--min-zoom', type=int, default=0, help='minimum zoom level (inclusive)')
    parser.add_argument('--max-zoom', type=int, default=5, help='maximum zoom level (inclusive)')
    parser.add_argument('--outdir', default='tiles', help='output directory')
    parser.add_argument('--overwrite', action='store_true', help='overwrite existing tiles')
    args = parser.parse_args()

    svg_path = args.svg
    outdir = args.outdir

    if not os.path.isfile(svg_path):
        raise SystemExit(f"SVG file not found: {svg_path}")

    for z in range(args.min_zoom, args.max_zoom + 1):
        tiles_per_side = 2 ** z
        width_px = TILE_SIZE * tiles_per_side
        print(f"Zoom {z}: rendering full PNG at {width_px}x{width_px}...")
        img = render_full_png(svg_path, width_px)
        print(f"  - rendered image size: {img.width}x{img.height}")

        if not args.overwrite:
            sample_tile = os.path.join(outdir, f"osm_100-l-1-{z}-0-0.png")
            if os.path.exists(sample_tile):
                print(f"  - tiles for zoom {z} already exist (use --overwrite to regenerate). Skipping.")
                continue

        print(f"  - splitting into {tiles_per_side}x{tiles_per_side} tiles...")
        make_tiles_from_image(img, z, outdir)
        print(f"  - zoom {z} done. Tiles saved under {outdir}")

    print("All done.")


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import sys

from PySide6.QtCore import QMimeData
from PySide6.QtWidgets import QApplication, QMainWindow, QPushButton


class ClipboardHelperWindow(QMainWindow):
    """
    A window to copy text with the secret hint
    """

    def __init__(self) -> None:
        super().__init__()
        self.setWindowTitle("ClipboardHelper")
        self.setGeometry(0, 0, 100, 100)

        self.button = QPushButton("Copy Secret", self)
        self.button.clicked.connect(self.copy_secret)
        self.setCentralWidget(self.button)

    def resizeEvent(self, event) -> None:
        print("button resized", file=sys.stderr)

    def copy_secret(self) -> None:
        mimeData = QMimeData()
        mimeData.setText("123456789test")
        mimeData.setData("x-kde-passwordManagerHint", b"secret")
        QApplication.clipboard().setMimeData(mimeData)

        mimeData = QMimeData()
        mimeData.setText("123456789")
        mimeData.setData("x-kde-passwordManagerHint", b"public")
        QApplication.clipboard().setMimeData(mimeData)
        self.hide()  # Move focus to plasmawindowed


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = ClipboardHelperWindow()
    window.showFullScreen()
    sys.exit(app.exec())



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT
# Idea inspired by Demitrius Belai's Java code: https://bugs.kde.org/attachment.cgi?id=170202
# See also: https://bugs.kde.org/show_bug.cgi?id=484647

# pylint: disable=global-statement

import os

# set_urgency_hint only works on X11
os.environ["GDK_BACKEND"] = "x11"
# Otherwise the following tests will fail
os.environ["NO_AT_BRIDGE"] = "1"
os.environ["GTK_A11Y"] = "none"
# Avoid temporary files
os.environ["GSETTINGS_BACKEND"] = "memory"
os.environ["GVFS_DISABLE_FUSE"] = "1"
os.environ["GIO_USE_VFS"] = "local"

import logging
from typing import Final, cast

import gi

gi.require_version("Gtk", "4.0")
gi.require_version("GdkX11", "4.0")
from gi.repository import GdkX11, Gio, GLib, Gtk

app: Gtk.Application | None = None
frame_1: Gtk.Window | None = None
frame_2: Gtk.Window | None = None
dialog_1: Gtk.Window | None = None
dbus_interface = None


class OrgKdeBug484647:
    """
    D-Bus interface for org.kde.bug484647
    """

    BUS_NAME: Final = "org.kde.bug484647"
    OBJECT_PATH: Final = "/484647"
    INTERFACE_NAME: Final = "org.kde.bug484647"

    connection: Gio.DBusConnection

    def __init__(self) -> None:
        self.reg_id: int = 0
        self.owner_id: int = Gio.bus_own_name(Gio.BusType.SESSION, self.BUS_NAME, Gio.BusNameOwnerFlags.NONE, self.on_bus_acquired, None, None)
        assert self.owner_id > 0

    def on_bus_acquired(self, connection: Gio.DBusConnection, name: str, *args) -> None:
        """
        The interface is ready, now register objects.
        """
        self.connection = connection
        introspection_data = Gio.DBusNodeInfo.new_for_xml("""
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN"
    "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node>
  <interface name="org.kde.bug484647">
    <method name="ChangeLeader1"/>
    <method name="ChangeLeader2"/>
    <method name="UnsetLeader"/>
    <method name="ShowTransientWindow"/>
    <method name="CloseTransientWindow"/>
    <method name="SetUrgencyHint"/>
    <method name="UnsetUrgencyHint"/>
  </interface>
</node>
""")
        self.reg_id = connection.register_object(self.OBJECT_PATH, introspection_data.interfaces[0], self.handle_method_call, None, None)
        assert self.reg_id > 0
        logging.info("interface registered")

    def handle_method_call(self, connection: Gio.DBusConnection, sender: str, object_path: str, interface_name: str, method_name: str, parameters: GLib.Variant, invocation: Gio.DBusMethodInvocation) -> None:
        assert dialog_1 is not None and frame_1 is not None
        logging.info("method call %s", method_name)

        if method_name == "ChangeLeader1":
            dialog_1.set_modal(True)
            dialog_1.set_transient_for(frame_1)
            invocation.return_value(None)
        elif method_name == "ChangeLeader2":
            dialog_1.set_modal(True)
            dialog_1.set_transient_for(frame_2)
            invocation.return_value(None)
        elif method_name == "UnsetLeader":
            dialog_1.set_modal(False)
            dialog_1.set_transient_for(None)
            invocation.return_value(None)
        elif method_name == "ShowTransientWindow":
            dialog_1.set_visible(True)
            invocation.return_value(None)
        elif method_name == "CloseTransientWindow":
            dialog_1.set_visible(False)
            invocation.return_value(None)
        elif method_name == "SetUrgencyHint":
            cast(GdkX11.X11Surface, dialog_1.get_surface()).set_urgency_hint(True)
            invocation.return_value(None)
        elif method_name == "UnsetUrgencyHint":
            cast(GdkX11.X11Surface, dialog_1.get_surface()).set_urgency_hint(False)
            invocation.return_value(None)
        else:
            invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.UNKNOWN_METHOD, f"Unknown method {method_name}")


def on_show(widget) -> None:
    global dialog_1
    dialog_1 = Gtk.Window(application=app, title="Dialog1")
    dialog_1.set_default_size(100, 100)
    dialog_1.set_modal(True)
    dialog_1.set_transient_for(widget)
    dialog_1.set_visible(True)


def on_activate(_app: Gtk.Application) -> None:
    global dbus_interface, frame_1, frame_2
    dbus_interface = OrgKdeBug484647()
    frame_1 = Gtk.Window(application=_app, title="Frame1")
    frame_2 = Gtk.Window(application=_app, title="Frame2")

    frame_1.set_default_size(200, 200)
    frame_2.set_default_size(200, 200)

    frame_2.connect("show", on_show)

    frame_1.set_visible(True)
    frame_2.set_visible(True)

    GLib.timeout_add_seconds(60, _app.quit)


if __name__ == "__main__":
    logging.getLogger().setLevel(logging.INFO)
    app = Gtk.Application(application_id='org.kde.testwindow')
    app.connect('activate', on_activate)
    app.run(None)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

import sys
import threading
from typing import Any, Final

from gi.repository import Gio, GLib


class GLibMainLoopThread(threading.Thread):

    def __init__(self) -> None:
        # Set up D-Bus loop
        self.loop = GLib.MainLoop()
        self.failSafeTimer = threading.Timer(600, self.loop.quit)

        # Create the thread
        super().__init__()

    def run(self) -> None:
        self.failSafeTimer.start()
        self.loop.run()

    def quit(self) -> None:
        self.failSafeTimer.cancel()
        self.loop.quit()


class OrgKdeKSplash:
    """
    D-Bus interfaces for org.kde.KSplash
    """

    BUS_NAME: Final = "org.kde.KSplash"
    OBJECT_PATH: Final = "/KSplash"

    connection: Gio.DBusConnection

    def __init__(self) -> None:
        self.reg_id: int = 0
        self.registered_event = threading.Event()
        self.owner_id: int = Gio.bus_own_name(Gio.BusType.SESSION, self.BUS_NAME, Gio.BusNameOwnerFlags.NONE, self.on_bus_acquired, None, None)
        assert self.owner_id > 0

        self.stage: str = ""
        self.stage_set_event = threading.Event()
        self.default_variant = GLib.Variant("(bnixqutdysgoa{sv}v)", [
            True,
            32767,
            2147483647,
            21474836470,
            65535,
            4294967295,
            18446744073709551615,
            1.23,
            255,
            "abc",
            "(bnixqutdysgoa{sv}v)",
            "/abc/def",
            {
                "int64": GLib.Variant("n", 32767),
                "objectPath": GLib.Variant("o", "/abc/def"),
                "string": GLib.Variant("s", "string"),
            },
            GLib.Variant("s", "variant"),
        ])
        self.read_write_prop = 1

    def quit(self) -> None:
        self.connection.unregister_object(self.reg_id)
        Gio.bus_unown_name(self.owner_id)
        self.connection.flush_sync(None)

    def on_bus_acquired(self, connection: Gio.DBusConnection, name: str, *args) -> None:
        """
        Interface is ready, now register objects.
        """
        self.connection = connection

        introspection_data = Gio.DBusNodeInfo.new_for_xml("""
<!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN"
    "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
<node>
  <interface name="org.kde.KSplash">
    <property name="readOnlyProp" type="(bnixqutdysgoa{sv}v)" access="read"/>
    <property name="readWriteProp" type="i" access="readwrite"/>
    <method name="setStage">
      <arg name="stage" type="s" direction="in"/>
    </method>
    <method name="updatePropertySilently"/>
    <method name="ping" />
    <signal name="pong">
      <arg name="arg1" type="b"/>
      <arg name="arg2" type="n"/>
      <arg name="arg3" type="i"/>
      <arg name="arg4" type="x"/>
      <arg name="arg5" type="q"/>
      <arg name="arg6" type="u"/>
      <arg name="arg7" type="t"/>
      <arg name="arg8" type="d"/>
      <arg name="arg9" type="y"/>
      <arg name="arg10" type="s"/>
      <arg name="arg11" type="g"/>
      <arg name="arg12" type="o"/>
      <arg name="arg13" type="a{sv}"/>
      <arg name="arg14" type="v"/>
    </signal>
    <method name="testAllTypes">
      <arg name="arg1" type="b" direction="in"/>
      <arg name="arg2" type="n" direction="in"/>
      <arg name="arg3" type="i" direction="in"/>
      <arg name="arg4" type="x" direction="in"/>
      <arg name="arg5" type="q" direction="in"/>
      <arg name="arg6" type="u" direction="in"/>
      <arg name="arg7" type="t" direction="in"/>
      <arg name="arg8" type="d" direction="in"/>
      <arg name="arg9" type="y" direction="in"/>
      <arg name="arg10" type="s" direction="in"/>
      <arg name="arg11" type="g" direction="in"/>
      <arg name="arg12" type="o" direction="in"/>
      <arg name="arg13" type="a{sv}" direction="in"/>
      <arg name="arg14" type="v" direction="in"/>
      <arg name="outArg" type="(bnixqutdysgoa{sv}v)" direction="out"/>
    </method>
    <method name="testListTypes">
      <arg name="arg1" type="ab" direction="in"/>
      <arg name="arg2" type="an" direction="in"/>
      <arg name="arg3" type="ai" direction="in"/>
      <arg name="arg4" type="ax" direction="in"/>
      <arg name="arg5" type="aq" direction="in"/>
      <arg name="arg6" type="au" direction="in"/>
      <arg name="arg7" type="at" direction="in"/>
      <arg name="arg8" type="ad" direction="in"/>
      <arg name="arg9" type="ay" direction="in"/>
      <arg name="arg10" type="as" direction="in"/>
      <arg name="arg11" type="ag" direction="in"/>
      <arg name="arg12" type="ao" direction="in"/>
      <arg name="arg13" type="aa{sv}" direction="in"/>
      <arg name="arg14" type="av" direction="in"/>
      <arg name="outArg" type="(abanaiaxaqauatadayasagaoaa{sv}av)" direction="out"/>
    </method>
  </interface>
</node>
""")
        self.reg_id = connection.register_object(self.OBJECT_PATH, introspection_data.interfaces[0], self.handle_method_call, self.handle_get_property, self.handle_set_property)
        assert self.reg_id > 0

        self.registered_event.set()

    def handle_method_call(self, connection: Gio.DBusConnection, sender: str, object_path: str, interface_name: str, method_name: str, parameters: GLib.Variant, invocation: Gio.DBusMethodInvocation) -> None:
        """
        Handles method calls for org.kde.KSplash
        """
        assert interface_name == self.BUS_NAME, f"Unknown interface {interface_name}"
        print(f"ksplash call {method_name}", file=sys.stderr, flush=True)

        if method_name == "setStage":
            if not parameters.is_of_type(GLib.VariantType("(s)")):
                invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.INVALID_ARGS, f"Incorrect signature {parameters.get_type_string()}")
                return
            self.stage = parameters.unpack()[0]
            self.stage_set_event.set()
            invocation.return_value(None)
        elif method_name == "updatePropertySilently":
            self.read_write_prop += 1
            invocation.return_value(None)
        elif method_name == "ping":
            Gio.DBusConnection.emit_signal(connection, None, object_path, interface_name, "pong", self.default_variant)
            invocation.return_value(None)
        elif method_name == "testAllTypes":
            if not parameters.is_of_type(GLib.VariantType("(bnixqutdysgoa{sv}v)")):
                invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.INVALID_ARGS, f"Incorrect signature {parameters.get_type_string()}")
                return
            # Check value types in the variant
            sv_map = parameters.get_child_value(12)
            first_entry_value = sv_map.get_child_value(0).get_child_value(1).get_child_value(0)
            second_entry_value = sv_map.get_child_value(1).get_child_value(1).get_child_value(0)
            third_entry_value = sv_map.get_child_value(2).get_child_value(1).get_child_value(0)
            if not first_entry_value.is_of_type(GLib.VariantType("n")) or not second_entry_value.is_of_type(GLib.VariantType("o")) or not third_entry_value.is_of_type(GLib.VariantType("s")):
                invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.INVALID_ARGS, f"Incorrect signature {first_entry_value.get_type_string()} {second_entry_value.get_type_string()} {third_entry_value.get_type_string()}")
                return
            invocation.return_value(GLib.Variant.new_tuple(self.default_variant))
        elif method_name == "testListTypes":
            if not parameters.is_of_type(GLib.VariantType("(abanaiaxaqauatadayasagaoaa{sv}av)")):
                invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.INVALID_ARGS, f"Incorrect signature {parameters.get_type_string()}")
                return
            ret = GLib.Variant("(abanaiaxaqauatadayasagaoaa{sv}av)", [
                [True],
                [32767],
                [2147483647],
                [21474836470],
                [65535],
                [4294967295],
                [18446744073709551615],
                [1.23],
                [255],
                ["abc"],
                ["(bnixqutdysgoa{sv}v)"],
                ["/abc/def"],
                [{
                    "int64": GLib.Variant("n", 32767),
                    "objectPath": GLib.Variant("o", "/abc/def"),
                    "string": GLib.Variant("s", "string"),
                }],
                [GLib.Variant("s", "variant")],
            ])
            invocation.return_value(GLib.Variant.new_tuple(ret))
        else:
            invocation.return_error_literal(Gio.dbus_error_quark(), Gio.DBusError.UNKNOWN_METHOD, f"Unknown method {method_name}")

    def handle_get_property(self, connection: Gio.DBusConnection, sender: str, object_path: str, interface_name: str, value: Any):
        print(f"ksplash get {value}", file=sys.stderr, flush=True)
        if value == "readOnlyProp":
            return self.default_variant
        if value == "readWriteProp":
            return GLib.Variant("i", self.read_write_prop)
        return None

    def handle_set_property(self, connection: Gio.DBusConnection, sender: str, object_path: str, interface_name: str, key: str, value: Any) -> bool:
        print(f"ksplash set {key} to {value}", file=sys.stderr, flush=True)
        if key == "readWriteProp":
            self.read_write_prop = value.get_int32()
            changed_properties = GLib.Variant('a{sv}', {
                "readWriteProp": GLib.Variant("i", self.read_write_prop),
            })
            Gio.DBusConnection.emit_signal(connection, None, object_path, "org.freedesktop.DBus.Properties", "PropertiesChanged", GLib.Variant.new_tuple(GLib.Variant("s", interface_name), changed_properties, GLib.Variant('as', ())))
        else:
            return False
        return True


if __name__ == '__main__':
    loop = GLibMainLoopThread()
    loop.start()
    dbus_interface = OrgKdeKSplash()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2024 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

from PySide6.QtWidgets import QApplication, QMainWindow, QMenu
from PySide6.QtCore import QTimer


class TestWindow(QMainWindow):

    def __init__(self, parent=None) -> None:
        super().__init__(parent)
        self.setWindowTitle("Test Window")

        menu_bar = self.menuBar()
        foo_menu = QMenu("&foo", self)
        bar_menu = QMenu("&bar", self)
        menu_bar.addMenu(foo_menu)
        menu_bar.addMenu(bar_menu)
        menu_bar.setVisible(False)

        QTimer.singleShot(60000, self.close)


if __name__ == "__main__":
    app = QApplication([])
    app.setDesktopFileName("org.kde.testwindow")
    win = TestWindow()
    win.show()
    app.exec()



# SPDX-FileCopyrightText: 2023 Fushan Wen <qydwhotmail@gmail.com>
# SPDX-License-Identifier: MIT

"""Common constants for Appium tests."""

APPIUM_SERVER_URL: str = "http://127.0.0.1:4723"
KDE_VERSION: int = 6
DEFAULT_IMPLICIT_TIMEOUT: int = 10000
DEFAULT_WAIT_TIMEOUT: int = 10
PROCESS_WAIT_ITERATIONS: int = 10
PROCESS_WAIT_INTERVAL: float = 1.0
